/*
  @file component/openmax_wfddmx_component/omx_wfddmx_component.c

  OpenMAX parser component. This component is a wfd packet parser that parses the input
  TS stream format so that client calls the appropriate decoder.

  Copyright (C) 2008  STMicroelectronics
  Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
  Copyright (C) 2015 Telechips Inc.

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 554 $
  Author $Author: B060934 $
  revised by B060955.
*/

#include <omxcore.h>
#include <omx_base_video_port.h>
#include <omx_base_audio_port.h>
#include <omx_base_clock_port.h>
#include <omx_wfddmx_component.h>
#include <OMX_TCC_Index.h>

#if defined(TCC_VPU_C5_INCLUDE)
#include <TCC_VPUs_CODEC.h> // VPU video codec
#endif

#if defined(TCC_VPU_C7_INCLUDE)
#include <TCC_VPU_CODEC.h> // VPU video codec
#endif

#if defined(TCC_VPU_D6_INCLUDE)
#include <TCC_VPU_CODEC.h> // VPU video codec
#endif

#include <sys/mman.h>
#include <dlfcn.h>

typedef cdk_core_t*(*tcc_cdk_get_CoreAddr)(void);
typedef int(*tcc_cdk_get_video_codec_type)(int);
typedef int(*tcc_cdk_get_audio_codec_type)(int);
typedef cdk_func_t *(*tcc_cdk_get_dmx_function)(unsigned int);

static tcc_cdk_get_CoreAddr sym_tcc_cdk_get_core_addr = NULL;
static tcc_cdk_get_audio_codec_type sym_tcc_cdk_get_audio_codec_type = NULL;
static tcc_cdk_get_video_codec_type sym_tcc_cdk_get_video_codec_type = NULL;
static tcc_cdk_get_dmx_function sym_tcc_cdk_get_dmx_function = NULL;

static void tcc_cdk_dmx_symbol_loading(void)
{  
    void *pModule = NULL;

    pModule = dlopen(LIBTCCCDKCORE_SO, RTLD_LAZY); 

    if(pModule == NULL)
        printf("Could not find libcdkcore.so.");

    sym_tcc_cdk_get_core_addr = dlsym(pModule,"TCC_CDK_GetCoreAddr");
    if(sym_tcc_cdk_get_core_addr == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_core_addr \n");

    sym_tcc_cdk_get_audio_codec_type = dlsym(pModule,"get_audio_codec_type");
    if(sym_tcc_cdk_get_audio_codec_type == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_audio_codec_type \n");

    sym_tcc_cdk_get_video_codec_type = dlsym(pModule,"get_video_codec_type");
    if(sym_tcc_cdk_get_video_codec_type == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_video_codec_type \n");

    sym_tcc_cdk_get_dmx_function = dlsym(pModule,"TCC_CDK_GetDMXFunc");
    if(sym_tcc_cdk_get_dmx_function == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_dmx_function \n");

    dlclose(pModule);
}


/*flags for the start time stamp for audio and video*/
static OMX_BOOL SyncFlag[2];

static pthread_mutex_t g_audio_con_mutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_mutex_t g_video_con_mutex = PTHREAD_MUTEX_INITIALIZER;
static void dump_frame(unsigned int frame_num,void * frame_addr,unsigned int frame_size)
{
		unsigned char *pFrameAddr;
		unsigned long ulCount=0;
		pFrameAddr = (unsigned char *)frame_addr;
		int i;

		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO,"Address = [%08x] \t    Size=[%d] \n",(unsigned int)frame_addr,frame_size );
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO,"********************** < %d CDK Demuxer Frame Dump Start > ********************\n",frame_num);

		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO," %x ",(unsigned int)pFrameAddr);
		for(i=0;i<frame_size;i++)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO,"%02X ",(unsigned int)*pFrameAddr);
			pFrameAddr++;
			ulCount++;

			if(ulCount ==8)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO,"- %x ",(unsigned int)pFrameAddr);
			}

			if(ulCount > 15)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO,"\n %x ",(unsigned int)pFrameAddr);
				ulCount=0;
			}

		}
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO,"\n*************************** < Dump End > **************************\n");
	//exit(1);
}

int set_thread_attr( int prio , pthread_attr_t *p_tattr)
{
	pthread_attr_t tattr;

	int ret = 0;
	int stacksize;
	struct sched_param param;


	/* initialized with default attributes */
	ret |= pthread_attr_init (p_tattr);
	ret |= pthread_attr_setinheritsched(p_tattr, PTHREAD_EXPLICIT_SCHED);
	ret |= pthread_attr_setscope(p_tattr, PTHREAD_SCOPE_SYSTEM);
	ret |= pthread_attr_setschedpolicy(p_tattr, SCHED_RR);

	/* safe to get existing scheduling param */
	ret |= pthread_attr_getschedparam (p_tattr, &param);

	/* set the priority; others are unchanged */
	param.__sched_priority = prio;

	/* setting the new scheduling param */
	ret |= pthread_attr_setschedparam (p_tattr, &param);

	return 1;
}

void _video_send_thread(void* param)
{

	OMX_COMPONENTTYPE* openmaxStandComp 							= (OMX_COMPONENTTYPE*)param;

	omx_base_component_PrivateType* omx_base_component_Private		= openmaxStandComp->pComponentPrivate;
	omx_base_source_PrivateType* omx_base_source_Private			= openmaxStandComp->pComponentPrivate;
	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private	= openmaxStandComp->pComponentPrivate;

	omx_base_PortType *pVideoPort;
	pVideoPort = (omx_base_PortType*)omx_wfddmx_component_Private->ports[VIDEO_PORT_INDEX];

	tsem_t* pOutputSem = pVideoPort->pBufferSem;
	queue_t* pOutputQueue = pVideoPort->pBufferQueue;

	OMX_BUFFERHEADERTYPE* pOutputBuffer = NULL;
	OMX_COMPONENTTYPE* target_component;
	OMX_BOOL isOutputBufferNeeded = OMX_TRUE;
	OMX_S32 outBufExchanged = 0;
	OMX_S32	ret;
	int		status;
	mblk_t	*pAvstream;
	
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"#################<<< Video Send Thread Start >>>##################### \n");

	while(omx_wfddmx_component_Private->vsend_running)
	{
		if(omx_wfddmx_component_Private->state == OMX_StateExecuting || 
			omx_wfddmx_component_Private->state == OMX_StatePause ||
			omx_wfddmx_component_Private->state == OMX_StateIdle || 
			omx_wfddmx_component_Private->state == OMX_TransStateLoadedToIdle)
		{
			/*Wait till the ports are being flushed*/
			pthread_mutex_lock(&omx_base_source_Private->flush_mutex);
			while( PORT_IS_BEING_FLUSHED(pVideoPort)) 
			{
			       printf("WFDDMX - Video Flush Start ...............\n");
				if(omx_wfddmx_component_Private->video_cond_wait == 1)
				{	
					pthread_mutex_lock(&g_video_con_mutex);
					pthread_cond_signal(&omx_wfddmx_component_Private->video_cond);
					pthread_mutex_unlock(&g_video_con_mutex);
				}
			
				pthread_mutex_unlock(&omx_base_source_Private->flush_mutex);

				if(isOutputBufferNeeded == OMX_FALSE) {
					//printf("WFDDMX - Video port ReturnBufferFunction \n");
					pVideoPort->ReturnBufferFunction(pVideoPort, pOutputBuffer);
					outBufExchanged--;
					pOutputBuffer = NULL;
					isOutputBufferNeeded = OMX_TRUE;
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Ports are flushing,so returning output buffer\n");
				}
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s signalling flush all condition \n", __func__);
				tsem_up(omx_base_source_Private->flush_all_condition);

                if(PORT_IS_BEING_FLUSHED(pVideoPort))
                {
                    printf("WFDDMX - video port tsem_down \n");
                    tsem_down(omx_base_source_Private->flush_condition);
                    printf("WFDDMX - video port flush done - flush_condition  Flushed Status %d \n", pVideoPort->bIsPortFlushed);
                }
                pthread_mutex_lock(&omx_base_source_Private->flush_mutex);
			}
			pthread_mutex_unlock(&omx_base_source_Private->flush_mutex);
#if 0
			/*No buffer to process. So wait here*/
			if((isOutputBufferNeeded == OMX_TRUE && pOutputSem->semval == 0)
				&& (omx_wfddmx_component_Private->state != OMX_StateLoaded
				&& omx_wfddmx_component_Private->state != OMX_StateInvalid))
			{
				//DEBUG(DEB_LEV_TCC_ERR, "Video Port Waiting for output buffer \n");
				tsem_down(omx_base_source_Private->bMgmtSem);
				//DEBUG(DEB_LEV_TCC_ERR, "Video Port Waiting for output buffer tsem_UP\n");
			}
#endif

			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Waiting for output buffer semval=%d \n",pOutputSem->semval);
			if(pOutputSem->semval > 0 && isOutputBufferNeeded == OMX_TRUE && omx_wfddmx_component_Private->fast_stop == OMX_FALSE)
			{
				tsem_down(pOutputSem);
				if(pOutputQueue->nelem>0)
				{
					outBufExchanged++;
					isOutputBufferNeeded = OMX_FALSE;
					pOutputBuffer = dequeue(pOutputQueue);
					if(pOutputBuffer == NULL){
						TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Had NULL output buffer!!\n",__func__);
						break;
					}
				}
			}
			
			if(omx_wfddmx_component_Private->fast_stop)
			{
				if(isOutputBufferNeeded == OMX_FALSE) 
				{
					pVideoPort->ReturnBufferFunction(pVideoPort, pOutputBuffer);
					outBufExchanged--;
					pOutputBuffer = NULL;
					isOutputBufferNeeded = OMX_TRUE;
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Ports are flushing,so returning output buffer\n");
				}
				if(omx_wfddmx_component_Private->video_cond_wait == 1)
				{	
					pthread_mutex_lock(&g_video_con_mutex);
					pthread_cond_signal(&omx_wfddmx_component_Private->video_cond);
					pthread_mutex_unlock(&g_video_con_mutex);
				}
			}
			
			if((isOutputBufferNeeded == OMX_FALSE) && (omx_wfddmx_component_Private->fast_stop == OMX_FALSE)) {
				if(pOutputBuffer->nFlags == OMX_BUFFERFLAG_EOS) {
					pOutputBuffer->nFlags = 0;
				}

RETRY_GET_VIDEO:
				if(omx_wfddmx_component_Private->video_stream_q->q_mcount == 0)
				{
					if(omx_wfddmx_component_Private->fast_stop == OMX_FALSE)
					{
						DEBUG (DEB_LEV_TCC,"video queue empty\n");
						omx_wfddmx_component_Private->video_cond_wait = 1;
						pthread_mutex_lock(&g_video_con_mutex);
						pthread_cond_wait(&omx_wfddmx_component_Private->video_cond, &g_video_con_mutex);
						pthread_mutex_unlock(&g_video_con_mutex);			
						omx_wfddmx_component_Private->video_cond_wait = 0;
					}
				}
	
				if(((omx_wfddmx_component_Private->video_sender_start == OMX_TRUE) || (omx_wfddmx_component_Private->Use_Buffering_Mode == OMX_FALSE))&&(omx_wfddmx_component_Private->video_stream_q->q_mcount > 0))
				{
					pthread_mutex_lock(&omx_wfddmx_component_Private->video_mutex); 
					pAvstream = getq((msgb_queue_t *)omx_wfddmx_component_Private->video_stream_q);
					pthread_mutex_unlock(&omx_wfddmx_component_Private->video_mutex);

#ifdef USE_TEE_SECURE_MEMORY
                    if(omx_wfddmx_component_Private->Display_debug_Msg == OMX_TRUE){
                        TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[OMX_WFDDMX] secure_type %d use_secured_memory %d \n",
                                                                            pAvstream->secure_type, 
                                                                            omx_wfddmx_component_Private->use_secured_memory);
                        TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[OMX_WFDDMX] InputCounter %d SecureInMapInfo_Index %d\n", 
                                                                            pAvstream->input_counter,
                                                                            omx_wfddmx_component_Private->SecureInMapInfo_Index);
                    }

					if((pAvstream->secure_type == OMX_TRUE) && (omx_wfddmx_component_Private->use_secured_memory == OMX_TRUE))
					{
                        pthread_mutex_lock(omx_wfddmx_component_Private->secure_video_path_mutex);
                        pthread_mutex_lock(&omx_wfddmx_component_Private->decypt_fuction_mutex);


                        omx_wfddmx_component_Private->decrpytion_func(0/*Video Stream Instance*/,	
                                                                    pAvstream->input_counter/*Input Counter*/,
                                                                    pAvstream->frame_size/*stream size*/,
                                                                    pAvstream->b_rptr/*Input*/,
                                                                    omx_wfddmx_component_Private->SecureInMapInfo[omx_wfddmx_component_Private->SecureInMapInfo_Index]/*Output*/
                                                                    );

                        pthread_mutex_unlock(&omx_wfddmx_component_Private->decypt_fuction_mutex);
                        pthread_mutex_unlock(omx_wfddmx_component_Private->secure_video_path_mutex);
                        pAvstream->frame_addr = omx_wfddmx_component_Private->SecureInMapInfo[omx_wfddmx_component_Private->SecureInMapInfo_Index];

                        omx_wfddmx_component_Private->SecureInMapInfo_Index++;
                        if(omx_wfddmx_component_Private->SecureInMapInfo_Index == 4)
                            omx_wfddmx_component_Private->SecureInMapInfo_Index = 0;
					}
#endif
					//reserved1 : timestamp,  reserved2 : packet size
					memcpy(pOutputBuffer->pBuffer, &pAvstream, 4);
					pOutputBuffer->nFilledLen = 4;	
					pOutputBuffer->nTimeStamp = (OMX_TICKS)pAvstream->timestamp* 1000;
                    //printf("pOutputBuffer->nTimeStamp  : %lld  pAvstream->frame_size : %d \n", pOutputBuffer->nTimeStamp, pAvstream->frame_size);
					/*Output Buffer has been produced or EOS. So, return output buffer and get new buffer*/
					if(pOutputBuffer->nFilledLen != 0)
					{
						pVideoPort->ReturnBufferFunction(pVideoPort, pOutputBuffer);
						outBufExchanged--;
						pOutputBuffer = NULL;
						isOutputBufferNeeded = OMX_TRUE;
					}
#if 0	
					if(omx_wfddmx_component_Private->video_stream_q->q_mcount > 10)
					{ 
						flushq_tcc((msgb_queue_t *)omx_wfddmx_component_Private->video_stream_q,
									omx_wfddmx_component_Private->video_stream_q->q_mcount - 10);

						if(omx_wfddmx_component_Private->isVideoSkip == OMX_FALSE)
						{
							(*(omx_wfddmx_component_Private->callbacks->EventHandler))(openmaxStandComp, 
																						NULL,
																						OMX_EventVendorVideoSkip,
																						OMX_TRUE, /*Skip Start*/
																						NULL,
																						NULL);

							omx_wfddmx_component_Private->isVideoSkip = OMX_TRUE;

							flushq_tcc((msgb_queue_t *)omx_wfddmx_component_Private->audio_stream_q,
							omx_wfddmx_component_Private->audio_stream_q->q_mcount - 5);
						}

					}
					else
					{
						if(omx_wfddmx_component_Private->isVideoSkip == OMX_TRUE)
						{
							(*(omx_wfddmx_component_Private->callbacks->EventHandler))(openmaxStandComp, 
																						NULL,
																						OMX_EventVendorVideoSkip,
																						OMX_FALSE, /*Skip Stop*/
																						NULL,
																						NULL);
							omx_wfddmx_component_Private->isVideoSkip = OMX_FALSE;
						}
					}
#endif
				}
			}
			else
				usleep (10*1000);
		}
		else
		{
			usleep(10000);
		}
	}
	printf("Exit Video Sender Thread!!! \n");
	flushq((msgb_queue_t *)omx_wfddmx_component_Private->video_stream_q, 0);
	pthread_exit((void*)0);
}


void _audio_send_thread(void* param)
{
	OMX_COMPONENTTYPE* openmaxStandComp = (OMX_COMPONENTTYPE*)param;

	omx_base_component_PrivateType* omx_base_component_Private		= openmaxStandComp->pComponentPrivate;
	omx_base_source_PrivateType* omx_base_source_Private			= openmaxStandComp->pComponentPrivate;
	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private	= openmaxStandComp->pComponentPrivate;

	omx_base_PortType *pAudioPort;
	pAudioPort = (omx_base_PortType*)omx_wfddmx_component_Private->ports[AUDIO_PORT_INDEX];
	tsem_t* pOutputSem = pAudioPort->pBufferSem;
	queue_t* pOutputQueue = pAudioPort->pBufferQueue;

	OMX_BUFFERHEADERTYPE* pOutputBuffer = NULL;
	OMX_COMPONENTTYPE* target_component;
	OMX_BOOL isOutputBufferNeeded = OMX_TRUE;
	OMX_S32 outBufExchanged = 0;
	OMX_S32	ret;
	mblk_t	*pAvstream;
	
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"#################<<< Audio Send Thread Start >>>##################### \n");

	while(omx_wfddmx_component_Private->asend_running)
	{
		if(omx_wfddmx_component_Private->state == OMX_StateExecuting ||
			omx_wfddmx_component_Private->state == OMX_StatePause ||
			omx_wfddmx_component_Private->state == OMX_StateIdle ||
			omx_wfddmx_component_Private->state == OMX_TransStateLoadedToIdle)

		{
			/*Wait till the ports are being flushed*/
			pthread_mutex_lock(&omx_base_source_Private->flush_mutex);
			while( PORT_IS_BEING_FLUSHED(pAudioPort)) 
			{
			       printf("WFDDMX - Audio Port Flush Start ...............\n");
				if(omx_wfddmx_component_Private->audio_cond_wait == 1)
				{	
					pthread_mutex_lock(&g_audio_con_mutex);
					pthread_cond_signal(&omx_wfddmx_component_Private->audio_cond);
					pthread_mutex_unlock(&g_audio_con_mutex);
				}
				
				pthread_mutex_unlock(&omx_base_source_Private->flush_mutex);
				if(isOutputBufferNeeded == OMX_FALSE) {
					//printf("WFDDMX - Audio port ReturnBufferFunction \n");
					pAudioPort->ReturnBufferFunction(pAudioPort, pOutputBuffer);
					outBufExchanged--;
					pOutputBuffer = NULL;
					isOutputBufferNeeded = OMX_TRUE;
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Ports are flushing,so returning output buffer\n");
				}
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s signalling flush all condition \n", __func__);
				tsem_up(omx_base_source_Private->flush_all_condition);
                            if(PORT_IS_BEING_FLUSHED(pAudioPort))
                            {
                                printf("WFDDMX - Audio port  tsem_down \n");
                                tsem_down(omx_base_source_Private->flush_condition);
                                printf("WFDDMX - Audio port flush done - flush_condition  Flushed Status %d \n", pAudioPort->bIsPortFlushed);
                            }
				pthread_mutex_lock(&omx_base_source_Private->flush_mutex);
			}
			pthread_mutex_unlock(&omx_base_source_Private->flush_mutex);
#if 0
			/*No buffer to process. So wait here*/
			if((isOutputBufferNeeded == OMX_TRUE && pOutputSem->semval == 0)
				&& (omx_wfddmx_component_Private->state != OMX_StateLoaded
				&& omx_wfddmx_component_Private->state != OMX_StateInvalid))
			{
				//DEBUG(DEB_LEV_TCC_ERR, "Audio Port Waiting for output buffer \n");
				tsem_down(omx_base_source_Private->bMgmtSem);
				//DEBUG(DEB_LEV_TCC_ERR, "Audio Port Waiting for output buffer tsem_UP\n");
			}
#endif
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Waiting for output buffer semval=%d \n",pOutputSem->semval);
			if(pOutputSem->semval > 0 && isOutputBufferNeeded == OMX_TRUE && omx_wfddmx_component_Private->fast_stop == OMX_FALSE )
			{
				tsem_down(pOutputSem);
				if(pOutputQueue->nelem>0)
				{
					outBufExchanged++;
					isOutputBufferNeeded = OMX_FALSE;
					pOutputBuffer = dequeue(pOutputQueue);
					if(pOutputBuffer == NULL){
						TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Had NULL output buffer!!\n",__func__);
						break;
					}
				}
			}
			if(omx_wfddmx_component_Private->fast_stop)
			{
				if(isOutputBufferNeeded == OMX_FALSE) 
				{
					pAudioPort->ReturnBufferFunction(pAudioPort, pOutputBuffer);
					outBufExchanged--;
					pOutputBuffer = NULL;
					isOutputBufferNeeded = OMX_TRUE;
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Ports are flushing,so returning output buffer\n");
				}
				if(omx_wfddmx_component_Private->audio_cond_wait == 1)
				{	
					pthread_mutex_lock(&g_audio_con_mutex);
					pthread_cond_signal(&omx_wfddmx_component_Private->audio_cond);
					pthread_mutex_unlock(&g_audio_con_mutex);
				}
			}
			
			if((isOutputBufferNeeded == OMX_FALSE) && (omx_wfddmx_component_Private->fast_stop == OMX_FALSE)){
				if(pOutputBuffer->nFlags == OMX_BUFFERFLAG_EOS) {
					pOutputBuffer->nFlags = 0;
				}

RETRY_GET_AUDIO:
				if(omx_wfddmx_component_Private->audio_stream_q->q_mcount == 0)
				{
					if(omx_wfddmx_component_Private->fast_stop == OMX_FALSE)
					{
						DEBUG (DEB_LEV_TCC, "audio queue empty\n");
						omx_wfddmx_component_Private->audio_cond_wait = 1;
						pthread_mutex_lock(&g_audio_con_mutex);
						pthread_cond_wait(&omx_wfddmx_component_Private->audio_cond, &g_audio_con_mutex);
						pthread_mutex_unlock(&g_audio_con_mutex);
						omx_wfddmx_component_Private->audio_cond_wait = 0;

						if(omx_wfddmx_component_Private->Use_Buffering_Mode == OMX_TRUE)
						{
							omx_wfddmx_component_Private->audio_sender_start = OMX_FALSE;
							omx_wfddmx_component_Private->video_sender_start = OMX_FALSE;
							(*(omx_wfddmx_component_Private->callbacks->EventHandler))(openmaxStandComp, 
																					NULL,
																					OMX_EventVendorBufferingStart,
																					NULL,
																					NULL,
																					NULL);
						}
					}
				}

				if(omx_wfddmx_component_Private->Use_Buffering_Mode == OMX_TRUE)
				{
					if(omx_wfddmx_component_Private->audio_stream_q->q_mcount > 25 && omx_wfddmx_component_Private->audio_sender_start == OMX_FALSE)				
					{	
						omx_wfddmx_component_Private->audio_sender_start = OMX_TRUE;
					}

                    if(omx_wfddmx_component_Private->audio_stream_q->q_mcount > 27)				
                    {	
                        flushq_tcc((msgb_queue_t *)omx_wfddmx_component_Private->audio_stream_q,
                        omx_wfddmx_component_Private->audio_stream_q->q_mcount - 27);
                    }
				}
				
				if((omx_wfddmx_component_Private->audio_sender_start == OMX_TRUE && omx_wfddmx_component_Private->audio_stream_q->q_mcount > 20) ||
					(omx_wfddmx_component_Private->Use_Buffering_Mode == OMX_FALSE))				
				{
					if((omx_wfddmx_component_Private->audio_stream_q->q_mcount > 0))
					{
						pthread_mutex_lock(&omx_wfddmx_component_Private->audio_mutex); 
						pAvstream = getq((msgb_queue_t *)omx_wfddmx_component_Private->audio_stream_q);
						pthread_mutex_unlock(&omx_wfddmx_component_Private->audio_mutex);

						//reserved1 : timestamp,  reserved2 : packet size
						memcpy(pOutputBuffer->pBuffer, &pAvstream, 4);		
						pOutputBuffer->nFilledLen = 4;
						pOutputBuffer->nTimeStamp = (OMX_TICKS)pAvstream->timestamp* 1000;
						/*Output Buffer has been produced or EOS. So, return output buffer and get new buffer*/
						if(pOutputBuffer->nFilledLen != 0)
						{
							pAudioPort->ReturnBufferFunction(pAudioPort, pOutputBuffer);
							outBufExchanged--;
							pOutputBuffer = NULL;
							isOutputBufferNeeded = OMX_TRUE;
						}
					}
				}
			}
			else
				usleep (100);
		}
		else
			usleep(10*1000);
	}
	printf("Exit Audio Sender Thread!!!\n");
	flushq((msgb_queue_t *)omx_wfddmx_component_Private->audio_stream_q, 0);
	pthread_exit((void*)0);
}

unsigned long long GetTime_MSec(void) // milli second
{
	struct timeval now;
	gettimeofday (&now, NULL);
	return ((unsigned long long) now.tv_sec) * 1000 + ((unsigned long long) now.tv_usec) / 1000;
}
int last_pts; 
void* _demuxer_thread(void* arg)
{
	OMX_COMPONENTTYPE* openmaxStandComp = (OMX_COMPONENTTYPE*)arg;
	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
	int	ret;
	unsigned long long start_time;
	unsigned long long end_time;

	cdk_func_t* dmx_func = omx_wfddmx_component_Private->dmx_func;

	while(omx_wfddmx_component_Private->dmx_running)
	{
		ret = dmx_func( CDMX_GETSTREAM, NULL, &omx_wfddmx_component_Private->sCDmxInput, &omx_wfddmx_component_Private->sCDmxOutput, NULL);
		if( ret < 0 )
		{
			DEBUG (DEB_LEV_TCC_ERR,"[OMX_CDKDMX] Err Get Stream !! Err:%d\n", ret);
			usleep(10*1000);

			if(omx_wfddmx_component_Private->video_cond_wait == 1)
			{	
				pthread_mutex_lock(&g_video_con_mutex);
				pthread_cond_signal(&omx_wfddmx_component_Private->video_cond);
				pthread_mutex_unlock(&g_video_con_mutex);
			}

			if(omx_wfddmx_component_Private->audio_cond_wait == 1)
			{	
				pthread_mutex_lock(&g_audio_con_mutex);
				pthread_cond_signal(&omx_wfddmx_component_Private->audio_cond);
				pthread_mutex_unlock(&g_audio_con_mutex);
			}
			printf("Exit Demuxer Thread!!! \n");
			pthread_exit((void*)0);
			return;
		}
		else
		{
            if(omx_wfddmx_component_Private->Display_debug_Msg == OMX_TRUE){
                TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[OMX_WFDDMX] Packet type %d Secure Type %d \n",
                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_iPacketType, 
                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_ulSecureType);
                TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[OMX_WFDDMX] PTS %d StreamCounter %d InputCounter %d\n", 
                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_iTimeStamp,
                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ulStreamCounter,
                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ullInputCounter);
            }
        
            /*Before decryption, hdcp2 stack must be initialized*/
            if(omx_wfddmx_component_Private->sCDmxOutput.m_ulSecureType == CDMX_SECURE_TYPE_HDCP)
            {  
                if((omx_wfddmx_component_Private->sCDmxOutput.m_iPacketType == AV_PACKET_AUDIO) && (omx_wfddmx_component_Private->hdcp2_audio_stack_init == OMX_FALSE))
                {                           
                    printf("[WFDDMX] HDCP2.x Stack Init for Audio Stream with stream counter %d\n", omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ulStreamCounter);
#ifdef USE_TEE_SECURE_MEMORY                    
                    pthread_mutex_lock(omx_wfddmx_component_Private->secure_video_path_mutex);
#endif
                    pthread_mutex_lock(&omx_wfddmx_component_Private->decypt_fuction_mutex);
                    omx_wfddmx_component_Private->hdcp2_stack_init_func(1/*StreamInstance Audio with 1*/, omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ulStreamCounter/*StreamCounter*/);
                    pthread_mutex_unlock(&omx_wfddmx_component_Private->decypt_fuction_mutex);
#ifdef USE_TEE_SECURE_MEMORY                     
                    pthread_mutex_unlock(omx_wfddmx_component_Private->secure_video_path_mutex);
#endif
                    omx_wfddmx_component_Private->hdcp2_audio_stack_init = OMX_TRUE; 
                }

                if((omx_wfddmx_component_Private->sCDmxOutput.m_iPacketType == AV_PACKET_VIDEO) && (omx_wfddmx_component_Private->hdcp2_video_stack_init == OMX_FALSE))
                {  
                    printf("[WFDDMX] HDCP2.x Stack Init for Video Stream with stream counter %d\n", omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ulStreamCounter);
#ifdef USE_TEE_SECURE_MEMORY                    
                    pthread_mutex_lock(omx_wfddmx_component_Private->secure_video_path_mutex);
#endif
                    pthread_mutex_lock(&omx_wfddmx_component_Private->decypt_fuction_mutex);
                    omx_wfddmx_component_Private->hdcp2_stack_init_func(0/*StreamInstance Video with 0*/, omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ulStreamCounter/*StreamCounter*/);
                    pthread_mutex_unlock(&omx_wfddmx_component_Private->decypt_fuction_mutex);
#ifdef USE_TEE_SECURE_MEMORY                     
                    pthread_mutex_unlock(omx_wfddmx_component_Private->secure_video_path_mutex);
#endif
                    omx_wfddmx_component_Private->hdcp2_video_stack_init = OMX_TRUE; 
                }
            }
			/*Notify that source device skip a video frame*/
			if((omx_wfddmx_component_Private->sCDmxOutput.m_iPacketType == AV_PACKET_VIDEO))
			{
#if 0
                if((omx_wfddmx_component_Private->sCDmxOutput.m_iTimeStamp - last_pts) > 35 )
                printf("Video Frame Drop last-video-pts:[%d], current-video-pts:[%d]  diff:[%d]\n",last_pts, 
                                                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_iTimeStamp, 
                                                                                                    (omx_wfddmx_component_Private->sCDmxOutput.m_iTimeStamp - last_pts));

                last_pts = omx_wfddmx_component_Private->sCDmxOutput.m_iTimeStamp;				
#endif
			}
			
			if(omx_wfddmx_component_Private->sCDmxOutput.m_iPacketSize > CDMX_OUTPUT_BUFFER_SIZE)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"[OMX_CDKDMX]: Buffer Overflow ~ [%d]!!!!! \n",omx_wfddmx_component_Private->sCDmxOutput.m_iPacketSize);

			}
			else
			{
				mblk_t *m = NULL;

				if((omx_wfddmx_component_Private->sCDmxOutput.m_ulSecureType == CDMX_SECURE_TYPE_HDCP)&&
#ifdef USE_TEE_SECURE_MEMORY /*for Noe TEE Test*/                   
				(omx_wfddmx_component_Private->sCDmxOutput.m_iPacketType == AV_PACKET_AUDIO) &&
#endif
				(omx_wfddmx_component_Private->decrpytion_func != NULL))
				{ 
                    m = allocb((omx_wfddmx_component_Private->sCDmxOutput.m_iPacketSize + 15) & 0xfffffff0, 0);
#ifdef USE_TEE_SECURE_MEMORY                    
                    pthread_mutex_lock(omx_wfddmx_component_Private->secure_video_path_mutex);
#endif
                    pthread_mutex_lock(&omx_wfddmx_component_Private->decypt_fuction_mutex);

                    unsigned int stream_instance = (omx_wfddmx_component_Private->sCDmxOutput.m_iPacketType == AV_PACKET_VIDEO) ? 0 : 1;

                    omx_wfddmx_component_Private->decrpytion_func(stream_instance/*StreamInstance*/,
                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ullInputCounter/*InputCounter*/,
                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_iPacketSize/*Size*/,
                                                                    omx_wfddmx_component_Private->sCDmxOutput.m_pPacketData/*Input*/,
                                                                    m->b_wptr/*Output*/);
                 
                    pthread_mutex_unlock(&omx_wfddmx_component_Private->decypt_fuction_mutex);
#ifdef USE_TEE_SECURE_MEMORY                     
                    pthread_mutex_unlock(omx_wfddmx_component_Private->secure_video_path_mutex);
#endif
                }
				else
				{
					/*Make message block*/
					m = allocb(omx_wfddmx_component_Private->sCDmxOutput.m_iPacketSize, 0);
					/*Copy A/V Stream*/
					memcpy(m->b_wptr, omx_wfddmx_component_Private->sCDmxOutput.m_pPacketData, omx_wfddmx_component_Private->sCDmxOutput.m_iPacketSize);
				}

				m->b_wptr		+= omx_wfddmx_component_Private->sCDmxOutput.m_iPacketSize;
				/*Set Timestamp*/
				m->timestamp	 = omx_wfddmx_component_Private->sCDmxOutput.m_iTimeStamp;
				/*Set Packet Size*/
				m->frame_size	 = omx_wfddmx_component_Private->sCDmxOutput.m_iPacketSize;
				/*Set Packet Discontinuous*/
				m->packet_discon = omx_wfddmx_component_Private->sCDmxOutput.m_bPacketDiscont;
				/*Set Secure Type*/
				m->secure_type	 = OMX_FALSE;
				/*Reset stream counter and input counter*/
				m->stream_counter = OMX_FALSE;
				m->input_counter  = OMX_FALSE;


				/*This is an encypted video stream, so Set the secure type and stream, input counter for decryption and secure video path*/		
				if(omx_wfddmx_component_Private->sCDmxOutput.m_ulSecureType == CDMX_SECURE_TYPE_HDCP)
				{
					m->secure_type	  = OMX_TRUE;
					m->stream_counter = omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ulStreamCounter;
					m->input_counter  =	omx_wfddmx_component_Private->sCDmxOutput.m_enSecureInfo.stHdcpInfo.ullInputCounter;
				}

                /*Packet Discontinuous occurs,	We need an IDR-Request event*/
				if(omx_wfddmx_component_Private->sCDmxOutput.m_bPacketDiscont == OMX_TRUE)
				{
					(*(omx_wfddmx_component_Private->callbacks->EventHandler))(openmaxStandComp, 
    																			NULL,
    																			OMX_EventVendorRequestIDRPicture,
    																			NULL,
    																			NULL,
    																			NULL);
					
					omx_wfddmx_component_Private->Detected_Packet_discontinuous = OMX_TRUE;
				}			
				
				/*Insert Video stream into Queue */
				if(omx_wfddmx_component_Private->sCDmxOutput.m_iPacketType == AV_PACKET_VIDEO)
				{     
					if(omx_wfddmx_component_Private->Detected_Packet_discontinuous == OMX_TRUE)
					{
						m->packet_discon = OMX_TRUE;
						omx_wfddmx_component_Private->Detected_Packet_discontinuous = OMX_FALSE;
					}

					pthread_mutex_lock(&omx_wfddmx_component_Private->video_mutex);
					putq(omx_wfddmx_component_Private->video_stream_q, m);		 
					pthread_mutex_unlock(&omx_wfddmx_component_Private->video_mutex);

					if(omx_wfddmx_component_Private->video_cond_wait == 1)
					{	
						pthread_mutex_lock(&g_video_con_mutex);
						pthread_cond_signal(&omx_wfddmx_component_Private->video_cond);
						pthread_mutex_unlock(&g_video_con_mutex);
					}
				}

				/*Insert Audio stream into Queue */
				if(omx_wfddmx_component_Private->sCDmxOutput.m_iPacketType == AV_PACKET_AUDIO)
				{
				    if(omx_wfddmx_component_Private->Use_Buffering_Mode == OMX_FALSE)
                        usleep(10*1000);
                    
                    pthread_mutex_lock(&omx_wfddmx_component_Private->audio_mutex);

                    if((omx_wfddmx_component_Private->Use_Buffering_Mode == OMX_FALSE)&&(omx_wfddmx_component_Private->audio_stream_q->q_mcount > 1))
                    { 
                        flushq_tcc((msgb_queue_t *)omx_wfddmx_component_Private->audio_stream_q,
                        omx_wfddmx_component_Private->audio_stream_q->q_mcount - 1);
                    }

                    putq(omx_wfddmx_component_Private->audio_stream_q, m);		 
                    pthread_mutex_unlock(&omx_wfddmx_component_Private->audio_mutex);

                    if(omx_wfddmx_component_Private->audio_cond_wait == 1)
                    {	
                        pthread_mutex_lock(&g_audio_con_mutex);
                        pthread_cond_signal(&omx_wfddmx_component_Private->audio_cond);
                        pthread_mutex_unlock(&g_audio_con_mutex);
                    }
				}		
			}	
		}
	}
	printf("Exit Demuxer Thread!!! \n");
	pthread_exit((void*)0);
	return;
}

OMX_ERRORTYPE omx_wfddmx_component_Init(OMX_COMPONENTTYPE *openmaxStandComp) {
	return (omx_wfddmx_component_Constructor(openmaxStandComp, WFDSINK_DMX_NAME));
}

/** The Constructor
 */
OMX_ERRORTYPE omx_wfddmx_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp,OMX_STRING cComponentName) {

  OMX_ERRORTYPE err = OMX_ErrorNone;
  omx_base_video_PortType *pPortV;
  omx_base_audio_PortType *pPortA;
  omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private;
  
  TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In %s \n",__func__);

  if (!openmaxStandComp->pComponentPrivate) {
	openmaxStandComp->pComponentPrivate = TCC_calloc(1, sizeof(omx_wfddmx_component_PrivateType));
	if(openmaxStandComp->pComponentPrivate == NULL) {
	  return OMX_ErrorInsufficientResources;
	}
  }

  /*Assign size of the derived port class,so that proper memory for port class can be allocated*/
  omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
  omx_wfddmx_component_Private->ports = NULL;


  err = omx_base_source_Constructor(openmaxStandComp, cComponentName);

  TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME," ******** %s %d\n",__func__,__LINE__);

  omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainAudio].nStartPortNumber = 0;
  omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts = 1;

  omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainVideo].nStartPortNumber = 1;
  omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts = 1;

 
   /** Allocate Ports and call port constructor. */
  if ((omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts  +
	   omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts ) && !omx_wfddmx_component_Private->ports)
	{
		omx_wfddmx_component_Private->ports = TCC_calloc((omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts  +
		omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts ), sizeof(omx_base_PortType *));
		if (!omx_wfddmx_component_Private->ports)
		{
			return OMX_ErrorInsufficientResources;
		}
		/* allocate video port*/
	   omx_wfddmx_component_Private->ports[VIDEO_PORT_INDEX] = TCC_calloc(1, sizeof(omx_base_video_PortType));
	   if (!omx_wfddmx_component_Private->ports[VIDEO_PORT_INDEX])
		   return OMX_ErrorInsufficientResources;
	   /* allocate audio port*/
	   omx_wfddmx_component_Private->ports[AUDIO_PORT_INDEX] = TCC_calloc(1, sizeof(omx_base_audio_PortType));
	   if (!omx_wfddmx_component_Private->ports[AUDIO_PORT_INDEX])
		   return OMX_ErrorInsufficientResources;
   }

  base_video_port_Constructor(openmaxStandComp, &omx_wfddmx_component_Private->ports[VIDEO_PORT_INDEX], VIDEO_PORT_INDEX, OMX_FALSE);
  base_audio_port_Constructor(openmaxStandComp, &omx_wfddmx_component_Private->ports[AUDIO_PORT_INDEX], AUDIO_PORT_INDEX, OMX_FALSE);

  pPortV = (omx_base_video_PortType *) omx_wfddmx_component_Private->ports[VIDEO_PORT_INDEX];
  pPortA = (omx_base_audio_PortType *) omx_wfddmx_component_Private->ports[AUDIO_PORT_INDEX];

  /*Input pPort buffer size is equal to the size of the output buffer of the previous component*/
  pPortV->sPortParam.nBufferSize = VIDEO_DEC_IN_BUFFER_SIZE; //DEFAULT_OUT_BUFFER_SIZE;
  pPortV->sPortParam.nBufferCountMin = 4;
  pPortV->sPortParam.nBufferCountActual = 4;

  pPortA->sPortParam.nBufferSize = AUDIO_DEC_IN_BUFFER_SIZE; //DEFAULT_IN_BUFFER_SIZE;
  pPortA->sPortParam.nBufferCountMin = 4;
  pPortA->sPortParam.nBufferCountActual = 4; 
  
  omx_wfddmx_component_Private->BufferMgmtCallback = omx_wfddmx_component_BufferMgmtCallback;
  omx_wfddmx_component_Private->BufferMgmtFunction = omx_base_source_twoport_BufferMgmtFunction;

  omx_wfddmx_component_Private->destructor = omx_wfddmx_component_Destructor;
  omx_wfddmx_component_Private->messageHandler = omx_wfddmx_component_MessageHandler;


  openmaxStandComp->SetParameter		= omx_wfddmx_component_SetParameter;
  openmaxStandComp->GetParameter		= omx_wfddmx_component_GetParameter;
  openmaxStandComp->SetConfig			= omx_wfddmx_component_SetConfig;
  openmaxStandComp->GetConfig			= omx_wfddmx_component_GetConfig;
  openmaxStandComp->GetExtensionIndex	= omx_wfddmx_component_GetExtensionIndex;

  /* Write in the default paramenters */
	omx_wfddmx_component_Private->avformatReady = OMX_FALSE;
	if(!omx_wfddmx_component_Private->avformatSyncSem)
	{
		omx_wfddmx_component_Private->avformatSyncSem = TCC_calloc(1,sizeof(tsem_t));
		if(omx_wfddmx_component_Private->avformatSyncSem == NULL) return OMX_ErrorInsufficientResources;
			tsem_init(omx_wfddmx_component_Private->avformatSyncSem, 0);
	}

	/*Default Coding type*/
	omx_wfddmx_component_Private->video_coding_type = OMX_VIDEO_CodingAVC;
	omx_wfddmx_component_Private->audio_coding_type = OMX_AUDIO_CodingAAC;
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"Out ******** %s \n",__func__);

	/*Telechips` Variables Initialization*/
	omx_wfddmx_component_Private->EndofStream = 0;

	//Initialization for CDK Demuxer
	memset(&omx_wfddmx_component_Private->sCDmxInfo,0,sizeof(cdmx_info_t));

	memset(&omx_wfddmx_component_Private->sCDmxInput,0,sizeof(cdmx_input_t));

	memset(&omx_wfddmx_component_Private->sCDmxOutput,0,sizeof(cdmx_output_t));

	memset(&omx_wfddmx_component_Private->sCDmxSeqHeader,0,sizeof(cdmx_seq_header_t));

	memset(&omx_wfddmx_component_Private->sCDmxSelStream,0,sizeof(cdmx_select_stream_t));

	memset(&omx_wfddmx_component_Private->sTempCDmxOutput,0,sizeof(cdmx_output_t));


    omx_wfddmx_component_Private->dmx_func= NULL;
    omx_wfddmx_component_Private->hdcp2_stack_init_func = NULL;
    omx_wfddmx_component_Private->decrpytion_func = NULL;
    omx_wfddmx_component_Private->hdcp2_audio_stack_init = OMX_FALSE; 
    omx_wfddmx_component_Private->hdcp2_video_stack_init = OMX_FALSE;
    omx_wfddmx_component_Private->SecuredInPmap.base =NULL;
    omx_wfddmx_component_Private->SecuredInPmap.size = 0;
    omx_wfddmx_component_Private->use_secured_memory = OMX_FALSE;

	setHeader(&omx_wfddmx_component_Private->sTimeStamp, sizeof(OMX_TIME_CONFIG_TIMESTAMPTYPE));
	omx_wfddmx_component_Private->sTimeStamp.nPortIndex = 0;
	omx_wfddmx_component_Private->sTimeStamp.nTimestamp = 0x7FFFFFFFFFFFFFFF;

	omx_wfddmx_component_Private->is_dmxinit = OMX_FALSE;
	omx_wfddmx_component_Private->fast_stop = OMX_FALSE;
	
	omx_wfddmx_component_Private->video_sender_start = OMX_FALSE; 
	omx_wfddmx_component_Private->audio_sender_start = OMX_FALSE;
	omx_wfddmx_component_Private->Detected_Packet_discontinuous = OMX_FALSE;
	omx_wfddmx_component_Private->Use_Buffering_Mode = OMX_FALSE;
    omx_wfddmx_component_Private->Display_debug_Msg = OMX_FALSE;

    tcc_cdk_dmx_symbol_loading();
	return err;
}

/* The Destructor */
OMX_ERRORTYPE omx_wfddmx_component_Destructor(OMX_COMPONENTTYPE *openmaxStandComp) {
  omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
  OMX_U32 i;

  if(omx_wfddmx_component_Private->avformatSyncSem) {
	tsem_deinit(omx_wfddmx_component_Private->avformatSyncSem);
	TCC_free(omx_wfddmx_component_Private->avformatSyncSem);
	omx_wfddmx_component_Private->avformatSyncSem=NULL;
  }

 
  /* frees port/s */
  if (omx_wfddmx_component_Private->ports) {
	for (i=0; i < (omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts  +
				   omx_wfddmx_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts); i++) {
	  if(omx_wfddmx_component_Private->ports[i])
		omx_wfddmx_component_Private->ports[i]->PortDestructor(omx_wfddmx_component_Private->ports[i]);
	}
	TCC_free(omx_wfddmx_component_Private->ports);
	omx_wfddmx_component_Private->ports=NULL;
  }

  TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In %s \n",__func__);
  return omx_base_source_Destructor(openmaxStandComp);
}

/** The Initialization function
 */
OMX_ERRORTYPE omx_wfddmx_component_Initialize(OMX_COMPONENTTYPE *openmaxStandComp) {

	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
	omx_base_video_PortType *pPortV;
	omx_base_audio_PortType *pPortA;
	int ret=0;
	int error=0;
	int codec_id;
	int iFourCC;
	int framerate_value;

	int i;
	unsigned long long measure;
	unsigned long long walltime, newtime;
	struct timeval	   tv;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In %s \n",__func__);


	cdk_func_t* dmx_func;
	cdk_core_t* pCdk;

	omx_wfddmx_component_Private->dmx_func = sym_tcc_cdk_get_dmx_function(omx_wfddmx_component_Private->container_type);

	dmx_func = omx_wfddmx_component_Private->dmx_func;

	pCdk = sym_tcc_cdk_get_core_addr();

	//Allocate Demuxer output buffer
	omx_wfddmx_component_Private->sCDmxInput.m_pPacketBuff = TCC_calloc(1,CDMX_OUTPUT_BUFFER_SIZE);
	omx_wfddmx_component_Private->sCDmxInput.m_iPacketBuffSize = CDMX_OUTPUT_BUFFER_SIZE;


	/* Setting the audio and video coding types of the audio and video ports based on the information obtained from the stream */
	/* for the video port */
	pPortV = (omx_base_video_PortType *) omx_wfddmx_component_Private->ports[VIDEO_PORT_INDEX];
	codec_id = sym_tcc_cdk_get_video_codec_type(omx_wfddmx_component_Private->sCDmxInfo.m_sVideoInfo.m_iFourCC);
	omx_wfddmx_component_Private->video_codec_type = codec_id;

	switch(codec_id)
	{
		case STD_AVC:
			pPortV->sPortParam.format.video.eCompressionFormat = OMX_VIDEO_CodingAVC;
			pPortV->sPortParam.format.video.nFrameWidth =omx_wfddmx_component_Private->sCDmxInfo.m_sVideoInfo.m_iWidth;
			pPortV->sPortParam.format.video.nFrameHeight =omx_wfddmx_component_Private->sCDmxInfo.m_sVideoInfo.m_iHeight;
			pPortV->sPortParam.format.video.xFramerate =omx_wfddmx_component_Private->sCDmxInfo.m_sVideoInfo.m_iFrameRate;
			omx_wfddmx_component_Private->video_coding_type = OMX_VIDEO_CodingAVC;
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"In %s Video Coding Type h.264\n",__func__);
			break;

		default :
		(*(omx_wfddmx_component_Private->callbacks->EventHandler))
		  (openmaxStandComp,
		  omx_wfddmx_component_Private->callbackData,
		  OMX_EventError, /* The command was completed */
		  OMX_ErrorFormatNotDetected, /* Format Not Detected */
		  VIDEO_PORT_INDEX, /* This is the output port index*/
		  NULL);
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"Trouble in %s No Video Coding Type Selected (only H264 and MPEG4 codecs supported)\n",__func__);
		return OMX_ErrorBadParameter;
	}

	/*
		the sequence header should be searched in the AVC,VC1,MPEG2 Streams included in TS,MPG(VOB) container.
	*/

	/* for the audio port*/
	pPortA = (omx_base_audio_PortType *) omx_wfddmx_component_Private->ports[AUDIO_PORT_INDEX];

	codec_id =sym_tcc_cdk_get_audio_codec_type(omx_wfddmx_component_Private->sCDmxInfo.m_sAudioInfo.m_iFormatId);
	switch(codec_id){
		case AUDIO_ID_AAC:
			pPortA->sPortParam.format.audio.eEncoding = OMX_AUDIO_CodingAAC;
			pPortA->sAudioParam.eEncoding = OMX_AUDIO_CodingAAC;
			omx_wfddmx_component_Private->audio_coding_type = OMX_AUDIO_CodingAAC;
			break;
		case AUDIO_ID_WAV:
			pPortA->sPortParam.format.audio.eEncoding = OMX_AUDIO_CodingPCM;
			pPortA->sAudioParam.eEncoding = OMX_AUDIO_CodingPCM;
			omx_wfddmx_component_Private->audio_coding_type = OMX_AUDIO_CodingPCM;
			break;
		default:
			pPortA->sPortParam.format.audio.eEncoding = OMX_AUDIO_CodingMP3;
			pPortA->sAudioParam.eEncoding = OMX_AUDIO_CodingMP3;
			omx_wfddmx_component_Private->audio_coding_type = OMX_AUDIO_CodingMP3;
	}  

	ret = 0;
	struct sched_param param;

	pthread_mutex_init(&omx_wfddmx_component_Private->video_mutex, NULL);
	pthread_mutex_init(&omx_wfddmx_component_Private->audio_mutex, NULL);
	pthread_mutex_init(&omx_wfddmx_component_Private->decypt_fuction_mutex, NULL);


	omx_wfddmx_component_Private->video_stream_q = (msgb_queue_t*)TCC_calloc(1,sizeof(msgb_queue_t));
	if(omx_wfddmx_component_Private->video_stream_q != NULL)
		qinit(omx_wfddmx_component_Private->video_stream_q);
	else
		return -1;

	omx_wfddmx_component_Private->audio_stream_q = (msgb_queue_t*)TCC_calloc(1,sizeof(msgb_queue_t));
	if(omx_wfddmx_component_Private->audio_stream_q != NULL)
		qinit(omx_wfddmx_component_Private->audio_stream_q);
	else
		return -1;

	/*Thread Function*/
	omx_wfddmx_component_Private->dmx_running =1;
	omx_wfddmx_component_Private->vsend_running =1;
	omx_wfddmx_component_Private->asend_running =1;
	omx_wfddmx_component_Private->isVideoSkip = OMX_FALSE;

	pthread_cond_init(&omx_wfddmx_component_Private->audio_cond, NULL);
	omx_wfddmx_component_Private->audio_cond_wait = 0;
	pthread_cond_init(&omx_wfddmx_component_Private->video_cond, NULL);
	omx_wfddmx_component_Private->video_cond_wait = 0;
	#if 0
	if(gMyDeviceInfo.settingInfo.wifi_info.broadcom==0) { // FIXME:
		printf("[OMX_DMX] USE_THREAD_MANAGEMENT- SCHED_RR\n");
		/*SCHED_RR */
		pthread_attr_t tattr;
		pthread_attr_t *p_tattr;

		p_tattr = &tattr;
		set_thread_attr(5,p_tattr);

		ret = pthread_create(&omx_wfddmx_component_Private->DmxThread,
																	 p_tattr,
																	 _demuxer_thread,
																	 openmaxStandComp);
		if(ret){
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Starting send buffer management thread failed\n");
			return OMX_ErrorUndefined;
		}
		pthread_attr_destroy(&tattr);

		p_tattr = &tattr;
		set_thread_attr(5,p_tattr);
		ret = pthread_create(&omx_wfddmx_component_Private->ASendThread,
																	 p_tattr,
																	 _audio_send_thread,
																	 openmaxStandComp);
		if(ret){
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Starting send buffer management thread failed\n");
			return OMX_ErrorUndefined;
		}
		pthread_attr_destroy(&tattr);

		p_tattr = &tattr;
		set_thread_attr(4,p_tattr);
		ret = pthread_create(&omx_wfddmx_component_Private->VSendThread,
																	 p_tattr,
																	 _video_send_thread,
																	 openmaxStandComp);
		if(ret){
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Starting send buffer management thread failed\n");
			return OMX_ErrorUndefined;
		}
		pthread_attr_destroy(&tattr);
	}else 
	#endif
	{
		//printf("[OMX_DMX] DON'T USE_THREAD_MANAGEMENT- SCHED_OTHER \n");
		/*SCHED_OTHER */
		ret = pthread_create(&omx_wfddmx_component_Private->DmxThread,
																	 NULL,
																	 _demuxer_thread,
																	 openmaxStandComp);
		if(ret){
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Starting send buffer management thread failed\n");
			return OMX_ErrorUndefined;
		}

		ret = pthread_create(&omx_wfddmx_component_Private->ASendThread,
																	 NULL,
																	 _audio_send_thread,
																	 openmaxStandComp);
		if(ret){
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Starting send buffer management thread failed\n");
			return OMX_ErrorUndefined;
		}

		ret = pthread_create(&omx_wfddmx_component_Private->VSendThread,
																	 NULL,
																	 _video_send_thread,
																	 openmaxStandComp);
		if(ret){
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Starting send buffer management thread failed\n");
			return OMX_ErrorUndefined;
		}
	}
	omx_wfddmx_component_Private->avformatReady = OMX_TRUE;
	/*Indicate that avformat is ready*/
	tsem_up(omx_wfddmx_component_Private->avformatSyncSem);
	return OMX_ErrorNone;
}

/** The DeInitialization function
 */
OMX_ERRORTYPE omx_wfddmx_component_Deinit(OMX_COMPONENTTYPE *openmaxStandComp) {

	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
	int ret;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Start...\n",__func__);

	omx_wfddmx_component_Private->dmx_running = 0;
	omx_wfddmx_component_Private->vsend_running = 0;
	omx_wfddmx_component_Private->asend_running = 0;
	if(omx_wfddmx_component_Private->video_cond_wait == 1)
	{	
		pthread_mutex_lock(&g_video_con_mutex);
		pthread_cond_signal(&omx_wfddmx_component_Private->video_cond);
		pthread_mutex_unlock(&g_video_con_mutex);
	}

	if(omx_wfddmx_component_Private->audio_cond_wait == 1)
	{	
		pthread_mutex_lock(&g_audio_con_mutex);
		pthread_cond_signal(&omx_wfddmx_component_Private->audio_cond);
		pthread_mutex_unlock(&g_audio_con_mutex);
	}	

	pthread_join(omx_wfddmx_component_Private->DmxThread,NULL);

	tsem_up(omx_wfddmx_component_Private->bMgmtSem);
	tsem_up(omx_wfddmx_component_Private->bMgmtSem);
	pthread_join(omx_wfddmx_component_Private->ASendThread,NULL);	
	pthread_join(omx_wfddmx_component_Private->VSendThread,NULL);
	pthread_mutex_destroy(&omx_wfddmx_component_Private->video_mutex);

	pthread_mutex_destroy(&omx_wfddmx_component_Private->audio_mutex);
	pthread_mutex_destroy(&omx_wfddmx_component_Private->decypt_fuction_mutex);

	TCC_free(omx_wfddmx_component_Private->video_stream_q);
	TCC_free(omx_wfddmx_component_Private->audio_stream_q);
	TCC_free(omx_wfddmx_component_Private->sCDmxInput.m_pPacketBuff);
#if 0//fdef USE_TEE_SECURE_MEMORY
	if(omx_wfddmx_component_Private->SecuredInPmap.base != NULL && omx_wfddmx_component_Private->SecuredInPmap.size != 0)
	{
		if(omx_wfddmx_component_Private->SecureInMapInfo[0]!= NULL)
			munmap((void*)omx_wfddmx_component_Private->SecureInMapInfo[0],2*1024*1024);
		if(omx_wfddmx_component_Private->SecureInMapInfo[1]!= NULL)
			munmap((void*)omx_wfddmx_component_Private->SecureInMapInfo[1],2*1024*1024);
		if(omx_wfddmx_component_Private->SecureInMapInfo[2]!= NULL)
			munmap((void*)omx_wfddmx_component_Private->SecureInMapInfo[2],2*1024*1024);
		if(omx_wfddmx_component_Private->SecureInMapInfo[3]!= NULL)
			munmap((void*)omx_wfddmx_component_Private->SecureInMapInfo[3],2*1024*1024);
		close(omx_wfddmx_component_Private->Tmem_fd);
	}
#endif
#ifdef USE_TEE_SECURE_MEMORY
	close(omx_wfddmx_component_Private->Tmem_fd);
#endif
	omx_wfddmx_component_Private->avformatReady = OMX_FALSE;
	tsem_reset(omx_wfddmx_component_Private->avformatSyncSem);

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Done...\n",__func__);

	return OMX_ErrorNone;
}

/**
 * This function processes the input file and returns packet by packet as an output data
 * this packet is used in audio/video decoder component for decoding
 */

void omx_wfddmx_component_BufferMgmtCallback(OMX_COMPONENTTYPE *openmaxStandComp, OMX_BUFFERHEADERTYPE* pOutputBuffer) {
	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
	omx_base_component_PrivateType* omx_base_component_Private=(omx_base_component_PrivateType*)openmaxStandComp->pComponentPrivate;
	omx_base_source_PrivateType* omx_base_source_Private = (omx_base_source_PrivateType*)omx_base_component_Private;

	int 								 error = 0;
	int 								 stream_index;
	int ret;
	omx_base_clock_PortType 			 *pClockPort;
	OMX_TIME_MEDIATIMETYPE* 			 pMediaTime;
	OMX_BUFFERHEADERTYPE*				 clockBuffer;
	
	/* don't move this routine!! this routine should be done first!! */
	if (omx_wfddmx_component_Private->avformatReady == OMX_FALSE) {
		if(omx_wfddmx_component_Private->state == OMX_StateExecuting) {
			/*wait for avformat to be ready*/
			tsem_down(omx_wfddmx_component_Private->avformatSyncSem);
		} else {
			return;
		}
	}

	pOutputBuffer->nFilledLen = 0;
	pOutputBuffer->nOffset = 0;
	pOutputBuffer->nFlags = 0;
}

OMX_ERRORTYPE omx_wfddmx_component_SetParameter(
	OMX_IN	OMX_HANDLETYPE hComponent,
	OMX_IN	OMX_INDEXTYPE nParamIndex,
	OMX_IN	OMX_PTR ComponentParameterStructure) {

	OMX_ERRORTYPE err = OMX_ErrorNone;
	OMX_VIDEO_PARAM_PORTFORMATTYPE *pVideoPortFormat;
	OMX_VIDEO_PARAM_MPEG2TYPE * pVideoMpeg2;
	OMX_AUDIO_PARAM_PORTFORMATTYPE *pAudioPortFormat;
	OMX_AUDIO_PARAM_MP3TYPE * pAudioMp3;
	OMX_U32 portIndex;
	OMX_U32 nFileNameLength;

	/* Check which structure we are being fed and make control its header */
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE*)hComponent;
	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
	omx_base_video_PortType* pVideoPort = (omx_base_video_PortType *) omx_wfddmx_component_Private->ports[OMX_BASE_SOURCE_OUTPUTPORT_INDEX];
	omx_base_audio_PortType* pAudioPort = (omx_base_audio_PortType *) omx_wfddmx_component_Private->ports[OMX_BASE_SOURCE_OUTPUTPORT_INDEX_1];

	if(ComponentParameterStructure == NULL) {
		return OMX_ErrorBadParameter;
	}

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Setting parameter %i\n", nParamIndex);

	switch(nParamIndex) {
		case OMX_IndexParamVideoPortFormat:
			pVideoPortFormat = (OMX_VIDEO_PARAM_PORTFORMATTYPE*)ComponentParameterStructure;
			portIndex = pVideoPortFormat->nPortIndex;
			/*Check Structure Header and verify component state*/
			err = omx_base_component_ParameterSanityCheck(hComponent, portIndex, pVideoPortFormat, sizeof(OMX_VIDEO_PARAM_PORTFORMATTYPE));
			if(err!=OMX_ErrorNone) {
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n",__func__,err);
				break;
			}
			if (portIndex < 1) {
				memcpy(&pVideoPort->sVideoParam,pVideoPortFormat,sizeof(OMX_VIDEO_PARAM_PORTFORMATTYPE));
			} else {
				return OMX_ErrorBadPortIndex;
			}
			break;
		case OMX_IndexParamAudioPortFormat:
			pAudioPortFormat = (OMX_AUDIO_PARAM_PORTFORMATTYPE*)ComponentParameterStructure;
			portIndex = pAudioPortFormat->nPortIndex;
			/*Check Structure Header and verify component state*/
			err = omx_base_component_ParameterSanityCheck(hComponent, portIndex, pAudioPortFormat, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE));
			if(err!=OMX_ErrorNone) {
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n",__func__,err);
				break;
			}
			if (portIndex == 1) {
				memcpy(&pAudioPort->sAudioParam,pAudioPortFormat,sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE));
			} else {
				return OMX_ErrorBadPortIndex;
			}
		break;
		
		case OMX_IndexVendorParamCDKDMXInfo:
		{
			memcpy(&omx_wfddmx_component_Private->sCDmxInfo, ComponentParameterStructure,sizeof(cdmx_info_t));
			break;
		}

		case OMX_IndexVendorSetHDCPStackInitFunc:
		{
			omx_wfddmx_component_Private->hdcp2_stack_init_func =
										 (unsigned long (*)(unsigned int, unsigned int))ComponentParameterStructure;
			break;
		}


		case OMX_IndexVendorSetHDCPDecryptionFunc:
		{
#ifdef HDCP2_SINK_SUPPORT
			omx_wfddmx_component_Private->decrpytion_func =
										 (unsigned long (*)(uint32_t , uint64_t , uint32_t , const uint8_t* , uint8_t*))ComponentParameterStructure;

#ifdef USE_TEE_SECURE_MEMORY
			omx_wfddmx_component_Private->Tmem_fd = open(TMEM_DEVICE, O_RDWR);
			if (omx_wfddmx_component_Private->Tmem_fd < 0) 
			{
				printf("can't open[%s] '%s'", strerror(errno), TMEM_DEVICE);
			}
			else 
			{
				pmap_get_info("secured_inbuff", &omx_wfddmx_component_Private->SecuredInPmap);
				if(omx_wfddmx_component_Private->SecuredInPmap.size > 0)
				{
#if 0				
					if( ( omx_wfddmx_component_Private->SecureInMapInfo[0] = (void*)mmap(0, 
																					omx_wfddmx_component_Private->SecuredInPmap.size, 
																					PROT_READ | PROT_WRITE, 
																					MAP_SHARED, 
																					omx_wfddmx_component_Private->Tmem_fd, 
																					omx_wfddmx_component_Private->SecuredInPmap.base) ) == MAP_FAILED ) {
						printf("%s device's secured_inbuffer's mmap failed.", TMEM_DEVICE);
					}
#endif
					omx_wfddmx_component_Private->SecureInMapInfo[0] = omx_wfddmx_component_Private->SecuredInPmap.base;
					omx_wfddmx_component_Private->SecureInMapInfo[1] = omx_wfddmx_component_Private->SecureInMapInfo[0] + 2*1024*1024;
					omx_wfddmx_component_Private->SecureInMapInfo[2] = omx_wfddmx_component_Private->SecureInMapInfo[1] + 2*1024*1024;
					omx_wfddmx_component_Private->SecureInMapInfo[3] = omx_wfddmx_component_Private->SecureInMapInfo[2] + 2*1024*1024;
					omx_wfddmx_component_Private->SecureInMapInfo_Index = 0;
				}
			}
#endif
#endif
			break;
		}

		case OMX_IndexVendorParamCDKContainerType:
		{		 
			unsigned int* pContainer_type;
			pContainer_type = (OMX_AUDIO_CONFIG_INFOTYPE*) ComponentParameterStructure;
			omx_wfddmx_component_Private->container_type =	*pContainer_type;//info->codec_type;
			break;
		}

		default: /*Call the base component function*/
			return omx_base_component_SetParameter(hComponent, nParamIndex, ComponentParameterStructure);
	}
	return err;
}

OMX_ERRORTYPE omx_wfddmx_component_GetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_INOUT OMX_PTR ComponentParameterStructure) {

  OMX_ERRORTYPE err = OMX_ErrorNone;
  OMX_PORT_PARAM_TYPE *pVideoPortParam, *pAudioPortParam;
  OMX_VIDEO_PARAM_PORTFORMATTYPE *pVideoPortFormat;
  OMX_VENDOR_EXTRADATATYPE sExtraData;
  OMX_AUDIO_PARAM_PORTFORMATTYPE *pAudioPortFormat;
  OMX_AUDIO_CONFIG_INFOTYPE * pAudioInfo;
  OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE*)hComponent;
  omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
  omx_base_video_PortType *pVideoPort = (omx_base_video_PortType *) omx_wfddmx_component_Private->ports[OMX_BASE_SOURCE_OUTPUTPORT_INDEX];
  omx_base_audio_PortType* pAudioPort = (omx_base_audio_PortType *) omx_wfddmx_component_Private->ports[OMX_BASE_SOURCE_OUTPUTPORT_INDEX_1];
  cdmx_info_t* cdmx_info;

  if (ComponentParameterStructure == NULL) {
	return OMX_ErrorBadParameter;
  }

  TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s Getting parameter %08x\n",__func__, nParamIndex);

  /* Check which structure we are being fed and fill its header */
  switch(nParamIndex) {

	default: /*Call the base component function*/
		return omx_base_component_GetParameter(hComponent, nParamIndex, ComponentParameterStructure);
	}
  return err;
}

/** This function initializes and deinitializes the library related initialization
  * needed for file parsing
  */
OMX_ERRORTYPE omx_wfddmx_component_MessageHandler(OMX_COMPONENTTYPE* openmaxStandComp,internalRequestMessageType *message) {
	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = (omx_wfddmx_component_PrivateType*)openmaxStandComp->pComponentPrivate;
	OMX_ERRORTYPE err = OMX_ErrorNone;
	OMX_STATETYPE oldState = omx_wfddmx_component_Private->state;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s\n", __func__);

	if (message->messageType == OMX_CommandStateSet)
	{
		if ((message->messageParam == OMX_StateExecuting) && (oldState == OMX_StateIdle))
		{
			omx_wfddmx_component_Private->is_need_splitter_thread = OMX_TRUE; 
			err = omx_wfddmx_component_Initialize(openmaxStandComp);
			if(err!=OMX_ErrorNone) {
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s  cdk dmx component Init Failed Error=%x\n",__func__,err);
				return err;
			}
			omx_wfddmx_component_Private->is_dmxinit = OMX_TRUE;
		}
		else if ((message->messageParam == OMX_StateLoaded) && (oldState == OMX_StateIdle))
		{
			if(omx_wfddmx_component_Private->is_dmxinit)
			{
				err = omx_wfddmx_component_Deinit(openmaxStandComp);
				if(err!=OMX_ErrorNone) {
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s  cdk dmx component DeInit Failed Error=%x\n",__func__,err);
					return err;
				}
				omx_wfddmx_component_Private->is_dmxinit = OMX_FALSE;
			}		
		}
		else if((message->messageParam == OMX_StateIdle) && (oldState == OMX_StateExecuting))
		{
	
		}
	}

	/* Execute the base message handling */
	err = omx_base_component_MessageHandler(openmaxStandComp,message);
  return err;
}

/** setting configurations */
OMX_ERRORTYPE omx_wfddmx_component_SetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure) {

	OMX_TIME_CONFIG_TIMESTAMPTYPE* sTimeStamp= (OMX_TIME_CONFIG_TIMESTAMPTYPE*)pComponentConfigStructure;
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;
	OMX_ERRORTYPE err = OMX_ErrorNone;
	omx_base_video_PortType *pPort;

	int ret=0;

	switch (nIndex) {

		case OMX_IndexVendorConfigFastStop:
		{
			OMX_AUDIO_CONFIG_INFOTYPE *info;
			info = (OMX_AUDIO_CONFIG_INFOTYPE*)pComponentConfigStructure;
			omx_wfddmx_component_Private->fast_stop = info->faststop;

			if(omx_wfddmx_component_Private->video_cond_wait == 1)
			{	
				pthread_mutex_lock(&g_video_con_mutex);
				pthread_cond_signal(&omx_wfddmx_component_Private->video_cond);
				pthread_mutex_unlock(&g_video_con_mutex);
			}

			if(omx_wfddmx_component_Private->audio_cond_wait == 1)
			{	
				pthread_mutex_lock(&g_audio_con_mutex);
				pthread_cond_signal(&omx_wfddmx_component_Private->audio_cond);
				pthread_mutex_unlock(&g_audio_con_mutex);
			}		  
		}
		break;
#ifdef USE_TEE_SECURE_MEMORY
		case OMX_IndexVendorSetUSESecuredMemoryForTEE:
		{
			omx_wfddmx_component_Private->use_secured_memory  = *(int *)pComponentConfigStructure;
		}
		break;

		case OMX_IndexVendorSetSecureVideoPathMutex:
		{
			omx_wfddmx_component_Private->secure_video_path_mutex = (pthread_mutex_t *)pComponentConfigStructure;
		}
		break;
#endif
		case OMX_IndexVendorSetVideoSenderStart:
		{	
           TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Video Sender Start \n");
			omx_wfddmx_component_Private->video_sender_start = OMX_TRUE;	
		}
		break;

		case OMX_IndexVendorSetBufferingMode:
		{
			if(omx_wfddmx_component_Private->Use_Buffering_Mode == OMX_FALSE)
			{
			    TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "A/V Sender Enable Buffering Mode \n");
				omx_wfddmx_component_Private->Use_Buffering_Mode = OMX_TRUE;	
			}
			else		
			{
			    TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "A/V Sender Disable Buffering Mode \n");
				omx_wfddmx_component_Private->Use_Buffering_Mode = OMX_FALSE;	
			}
		}
		break;

        case OMX_IndexVendorSetPlayerDebugMode:
            TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Enable Player Debug Msg. \n");
            omx_wfddmx_component_Private->Display_debug_Msg = OMX_TRUE;
        break;

        default: // delegate to superclass
		return omx_base_component_SetConfig(hComponent, nIndex, pComponentConfigStructure);
	}
	return OMX_ErrorNone;
}

/** getting configurations */
OMX_ERRORTYPE omx_wfddmx_component_GetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure) {

	OMX_VENDOR_EXTRADATATYPE sExtraData;
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
	omx_wfddmx_component_PrivateType* omx_wfddmx_component_Private = openmaxStandComp->pComponentPrivate;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In  %s OMX_IndexVendorVideoExtraData\n",__func__);
	switch (nIndex)
	{
		default: // delegate to superclass
		return omx_base_component_GetConfig(hComponent, nIndex, pComponentConfigStructure);

	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In  %s OMX_ErrorNone\n",__func__);
	return OMX_ErrorNone;
}

OMX_ERRORTYPE omx_wfddmx_component_GetExtensionIndex(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_STRING cParameterName,
  OMX_OUT OMX_INDEXTYPE* pIndexType) {

  TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In  %s \n",__func__);

  if(strcmp(cParameterName,"OMX.tcc.index.param.parser.inputfilename") == 0) {
	*pIndexType = OMX_IndexVendorParserInputFilename;
  } else if(strcmp(cParameterName,"OMX.tcc.index.config.videoextradata") == 0) {
	*pIndexType = OMX_IndexVendorVideoExtraData;
  } else if(strcmp(cParameterName,"OMX.tcc.index.config.audioextradata") == 0) {
	*pIndexType = OMX_IndexVendorAudioExtraData;
  }else if(strcmp(cParameterName,TCC_AUDIO_PARSER_MEDIA_STRING) == 0){
	*pIndexType = OMX_IndexVendorParamMediaInfo;
  } else {
	return OMX_ErrorBadParameter;
  }
  return OMX_ErrorNone;
}
