/**
  @file component/openmax_wfddmx_component/omx_wfddmx_component.h

  OpenMAX parser component. This component is a wfd packet parser that parses the input
  TS stream format so that client calls the appropriate decoder.

  Copyright (C) 2008  STMicroelectronics
  Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
  Copyright (C) 2015 Telechips Inc.

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 554 $
  Author $Author: B060934 $
  revised by B060955.

*/

#ifndef _OMX_WFDDMX_COMPONENT_H_
#define _OMX_WFDDMX_COMPONENT_H_

#include <OMX_Types.h>
#include <OMX_Component.h>
#include <OMX_Core.h>
#include <omx_base_source.h>
#include <string.h>
#include "tcc_file.h"
#include "tcc_memory.h"
#include "tcc_virtual_mem.h"

/* Specific include files for ffmpeg library related decoding*/

#include <tcc_video_common.h>

#include "cdk.h"
#include "cdmx.h"
#include "cmux.h"
#include "cdk_audio.h"
#include "pmap.h"
#include <str_utils.h>

#define OMX_CDK_DMX_SELECTIVE	0
#define OMX_CDK_DMX_SEQUENCIAL	1

/*>>>For DMX STREAM MODE*/

#define REVERSE_ORDER_CODE

#define CDMX_OUTPUT_BUFFER_SIZE 2*1024*1024

#define TMEM_DEVICE	"/dev/tmem"


typedef unsigned long (*hdcp2_stack_init_cb_func)(unsigned int StreamInstanceNr, unsigned int StreamCtr);
typedef unsigned long (*hdcp_decryption_cb_func)(uint32_t streamInstanceNr, uint64_t inputCtr, uint32_t dataSize, const uint8_t * inData, uint8_t * outData);


/*<<<For DMX STREAM MODE*/

/**
 * Identifies the syntax and semantics of the bitstream.
 * The principle is roughly:
 * Two decoders with the same ID can decode the same streams.
 * Two encoders with the same ID can encode compatible streams.
 * There may be slight deviations from the principle due to implementation
 * details.
 *
 * If you add a codec ID to this list, add it so that
 * 1. no value of a existing codec ID changes (that would break ABI),
 * 2. it is as close as possible to similar codecs.
 */

/** ts Parser component private structure.
 * see the define above
 * @param sTimeStamp Store Time Stamp to be set
 * @param avformatcontext is the ffmpeg video format context
 * @param avformatparameters is the ffmpeg video format related parameters 
 * @param tsnputformat is the ffmpeg video format related settings 
 * @param pTmpOutputBuffer is a temporary buffer to hold the data from av_read_frame 
 * @param sInputFileName is the input filename provided by client 
 * @param video_coding_type is the coding type determined by input file 
 * @param audio_coding_type is the coding type determined by input file 
 * @param semaphore for avformat syncrhonization 
 * @param avformatReady boolean flag that is true when the video format has been initialized 
 * @param xScale the scale of the media clock
 */
DERIVEDCLASS(omx_wfddmx_component_PrivateType, omx_base_source_PrivateType)
#define omx_wfddmx_component_PrivateType_FIELDS omx_base_source_PrivateType_FIELDS \
  OMX_TIME_CONFIG_TIMESTAMPTYPE       sTimeStamp; \
  OMX_U32       video_coding_type; \
  OMX_U32       audio_coding_type; \
  tsem_t*       avformatSyncSem; \
  OMX_BOOL      avformatReady; \
  OMX_U32		EndofStream; \
  cdmx_info_t	sCDmxInfo; \
  cdmx_output_t sCDmxOutput; \
  cdmx_input_t  sCDmxInput; \
  cdmx_seek_t	sCdmxSeek; \
  cdmx_output_t sTempCDmxOutput; \
  cdmx_seq_header_t sCDmxSeqHeader; \
  cdmx_select_stream_t sCDmxSelStream; \
  cdk_func_t*	dmx_func; \
  OMX_S32       container_type; \
  OMX_S32       video_codec_type; \
  OMX_U32       audio_ready; \  
  pthread_mutex_t video_mutex; \
  pthread_mutex_t audio_mutex; \
  pthread_mutex_t decypt_fuction_mutex; \
  pthread_mutex_t *secure_video_path_mutex; \
  pthread_cond_t  audio_cond; \
  OMX_BOOL        audio_cond_wait; \
  pthread_cond_t  video_cond; \
  OMX_BOOL        video_cond_wait; \ 
  OMX_U32       is_dmxinit; \
  OMX_U32       isVideoSkip; \  
  OMX_U32       fast_stop; \
  OMX_BOOL      dmx_running; \
  OMX_BOOL      vsend_running; \
  OMX_BOOL      asend_running; \
  pthread_t     DmxThread; \
  pthread_t     VSendThread; \
  pthread_t     ASendThread; \
  msgb_queue_t* video_stream_q; \
  OMX_BOOL 		use_secured_memory; \
  OMX_BOOL	    video_sender_start; \
  OMX_BOOL      audio_sender_start; \
  OMX_BOOL      hdcp2_audio_stack_init; \
  OMX_BOOL      hdcp2_video_stack_init; \
  hdcp2_stack_init_cb_func hdcp2_stack_init_func; \
  hdcp_decryption_cb_func  decrpytion_func; \
  int32_t       Tmem_fd; \
  pmap_t        SecuredInPmap; \
  void*         SecureInMapInfo[4]; \ 
  OMX_U32		SecureInMapInfo_Index; \
  OMX_BOOL      Detected_Packet_discontinuous; \
  OMX_BOOL      Use_Buffering_Mode; \
  OMX_BOOL      Display_debug_Msg; \  
  msgb_queue_t*  audio_stream_q;
ENDCLASS(omx_wfddmx_component_PrivateType)

/*  OMX_U32		GetOneFrameMode; \  
  OMX_U32		GetOneFrameTime;  \
  OMX_U32		GetOneFrameAfterSeek;  \*/


/* Component private entry points declaration */
OMX_ERRORTYPE omx_wfddmx_component_Init(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfddmx_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp,OMX_STRING cComponentName);
OMX_ERRORTYPE omx_wfddmx_component_Destructor(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfddmx_component_MessageHandler(OMX_COMPONENTTYPE*,internalRequestMessageType*);
OMX_ERRORTYPE omx_wfddmx_component_Initialize(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfddmx_component_Deinit(OMX_COMPONENTTYPE *openmaxStandComp);

void omx_wfddmx_component_BufferMgmtCallback(
  OMX_COMPONENTTYPE *openmaxStandComp,
  OMX_BUFFERHEADERTYPE* outputbuffer);

OMX_ERRORTYPE omx_wfddmx_component_GetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_INOUT OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfddmx_component_SetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_IN  OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfddmx_component_SetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure);

OMX_ERRORTYPE omx_wfddmx_component_GetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure);

OMX_ERRORTYPE omx_wfddmx_component_GetExtensionIndex(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_STRING cParameterName,
  OMX_OUT OMX_INDEXTYPE* pIndexType);

#endif /*_OMX_WFDDMX_COMPONENT_H_*/
