/**
  @file component/openmax_wfdfbdev_component/omx_wfdfbdev_component.c
  

  OpenMAX FBDEV sink component. This component is a video sink that copies
  data to a Linux framebuffer device.

  Originally developed by Peter Littlefield
  Copyright (C) 2007-2008  STMicroelectronics and Agere Systems
  Copyright (C) 2007-2008 Nokia Corporation and/or its subsidiary(-ies)
  Copyright (C) 2009-2010 Telechips Inc

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 554 $
  Author $Author: B060934 $
  revised by B060955.
*/

#include <errno.h>
#include <omxcore.h>
#include <omx_wfdfbdev_sink_component.h>

#include "tcc_virtual_mem.h"
#include <OMX_TCC_Index.h>
#include <OMX_Other.h>
#include "tcc_image_common.h"
#include "tccfb_ioctrl.h"
#include "tcc_scaler_interface.h"
#include "tcc_composite_interface.h"
#include "tcc_overlay_interface.h"
#include "tcc_fb_interface.h"

#define	NUMBER_OF_BUFFER		3

#define VSYNC_REINIT_ON_SEEKING

static int	gLCDC_KernelAddress[3];

static int	glcd_double_buf_num;
static int	glcd_double_buf_address[NUMBER_OF_BUFFER];
static int	glcd_double_buf_index;


/* TC TVOUT Option Enable */
/** height offset - reqd tadjust the display position - at the centre of upper half of screen */
#define HEIGHT_OFFSET 10

/** we assume, frame rate = 25 fps ; so one frame processing time = 40000 us */
/** we assume, frame rate = 30 fps ; so one frame processing time = 33333 us */
static OMX_U32 nFrameProcessTime = 33333;				   // in micro second

typedef struct tagImageInfo
{
	char* addr_y;
	char* addr_u;
	char* addr_v;
	unsigned long x;
	unsigned long y;
	unsigned long w;
	unsigned long h;
	int format;
}image_info_t;

typedef struct tagOverlayConfig
{
	unsigned int sx;
	unsigned int sy;
	unsigned int width;
	unsigned int height;
	unsigned int dispWidth;
	unsigned int dispHeight;
	unsigned int format;
	unsigned int transform;
}overlay_configure_t;

#define OVERLAY_SET_BUFFER 300

typedef struct tagOutmodeImageInfo
{
	int addr_y;
	int addr_u;
	int addr_v;
	unsigned int x_offset;
	unsigned int y_offset;
	unsigned int input_src_width;
	unsigned int input_src_height;
	unsigned int input_dest_width;
	unsigned int input_dest_height;
}outmode_image_info_t;

PERF_MEASURE_CHECK_GLOBAL_DEFINE(gs_PerfMeasureFBDEV);


//static int		gCapture_width;
//static int		gCapture_height;
//static int		gCapture_rotation;

static OMX_TICKS gPrevTime = 0;

//static unsigned int first_panscan_cmd=0;
#if 0
static unsigned int 	fb_window_pre[4] = {0, };
static unsigned int 	fb_addroffset_pre[2] = {0, };
#endif
#define CROP_MAX(val, limit)		((val) > (limit) ? (limit) : (val))
#define CROP_MIN(val, limit)		((val) < (limit) ? (limit) : (val))

//from vdec_k.c
extern int GetCodecInfoInterlaced(void);
extern int GetCodecInfoOddFirst(void);

static int get_framebuffer_address(void)
{
	int addr = glcd_double_buf_address[glcd_double_buf_index];

	// toggle double buffer index
	glcd_double_buf_index++;
	if(glcd_double_buf_index==glcd_double_buf_num)
	{
		glcd_double_buf_index = 0;
	}

	return addr;
}

OMX_S32 calcStride2 (omx_wfdfbdev_sink_component_PrivateType * omx_wfdfbdev_sink_component_Private)
{
	OMX_U32   stride;

	if (omx_wfdfbdev_sink_component_Private->vscr_info.bits_per_pixel == 32)
	{
		stride = omx_wfdfbdev_sink_component_Private->fscr_info.line_length;
	}
	else if (omx_wfdfbdev_sink_component_Private->vscr_info.bits_per_pixel == 8)
	{
		stride = omx_wfdfbdev_sink_component_Private->fscr_info.line_length * 4;
	}
	else
	{
		stride = omx_wfdfbdev_sink_component_Private->fscr_info.line_length *
			omx_wfdfbdev_sink_component_Private->vscr_info.bits_per_pixel / 8;
	}
	return stride;
}

/**  This function takes two inputs -
  * @param width is the input picture width
  * @param omx_pxlfmt is the input openmax standard pixel format
  * It calculates the byte per pixel needed to display the picture with the input omx_pxlfmt
  * The output stride for display is thus omx_wfdfbdev_sink_component_Private->product of input width and byte per pixel
  */
OMX_S32 fb_calcStride (OMX_U32 width, OMX_COLOR_FORMATTYPE omx_pxlfmt)
{
	OMX_U32   stride;
	OMX_U32   bpp;										   // bit per pixel

	switch (omx_pxlfmt)
	{
		case OMX_COLOR_FormatMonochrome:
			bpp = 1;
			break;
		case OMX_COLOR_FormatL2:
			bpp = 2;
		case OMX_COLOR_FormatL4:
			bpp = 4;
			break;
		case OMX_COLOR_FormatL8:
		case OMX_COLOR_Format8bitRGB332:
		case OMX_COLOR_FormatRawBayer8bit:
		case OMX_COLOR_FormatRawBayer8bitcompressed:
			bpp = 8;
			break;
		case OMX_COLOR_FormatRawBayer10bit:
			bpp = 10;
			break;
		case OMX_COLOR_FormatYUV411Planar:
		case OMX_COLOR_FormatYUV411PackedPlanar:
		case OMX_COLOR_Format12bitRGB444:
		case OMX_COLOR_FormatYUV420Planar:
		case OMX_COLOR_FormatYUV420PackedPlanar:
		case OMX_COLOR_FormatYUV420SemiPlanar:
		case OMX_COLOR_FormatYUV444Interleaved:
			bpp = 12;
			break;
		case OMX_COLOR_FormatL16:
		case OMX_COLOR_Format16bitARGB4444:
		case OMX_COLOR_Format16bitARGB1555:
		case OMX_COLOR_Format16bitRGB565:
		case OMX_COLOR_Format16bitBGR565:
		case OMX_COLOR_FormatYUV422Planar:
		case OMX_COLOR_FormatYUV422PackedPlanar:
		case OMX_COLOR_FormatYUV422SemiPlanar:
		case OMX_COLOR_FormatYCbYCr:
		case OMX_COLOR_FormatYCrYCb:
		case OMX_COLOR_FormatCbYCrY:
		case OMX_COLOR_FormatCrYCbY:
			bpp = 16;
			break;
		case OMX_COLOR_Format18bitRGB666:
		case OMX_COLOR_Format18bitARGB1665:
			bpp = 18;
			break;
		case OMX_COLOR_Format19bitARGB1666:
			bpp = 19;
			break;
		case OMX_COLOR_FormatL24:
		case OMX_COLOR_Format24bitRGB888:
		case OMX_COLOR_Format24bitBGR888:
		case OMX_COLOR_Format24bitARGB1887:
			bpp = 24;
			break;
		case OMX_COLOR_Format25bitARGB1888:
			bpp = 25;
			break;
		case OMX_COLOR_FormatL32:
		case OMX_COLOR_Format32bitBGRA8888:
		case OMX_COLOR_Format32bitARGB8888:
			bpp = 32;
			break;
		default:
			bpp = 0;
			break;
	}
	stride = (width * bpp) >> 3;	// division by 8 to get byte per pixel value
	return (OMX_S32) stride;
}

OMX_ERRORTYPE omx_wfdfbdev_sink_component_Init (OMX_COMPONENTTYPE * openmaxStandComp)
{
	return (omx_wfdfbdev_sink_component_Constructor (openmaxStandComp, WFDSINK_FBSINK_NAME));
}

/** The Constructor
 *
 * @param openmaxStandComp is the handle to be constructed
 * @param cComponentName is the name of the constructed component
 *
 */
OMX_ERRORTYPE omx_wfdfbdev_sink_component_Constructor (OMX_COMPONENTTYPE * openmaxStandComp, OMX_STRING cComponentName)
{
	OMX_ERRORTYPE err = OMX_ErrorNone;
	omx_wfdfbdev_sink_component_PortType *pPort;
	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private;

	if (!openmaxStandComp->pComponentPrivate)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In %s, allocating component\n", __func__);
		openmaxStandComp->pComponentPrivate = TCC_calloc (1, sizeof (omx_wfdfbdev_sink_component_PrivateType));
		if (openmaxStandComp->pComponentPrivate == NULL)
		{
			return OMX_ErrorInsufficientResources;
		}
	}
	else
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In %s, Error Component %x Already Allocated\n", __func__, (int) openmaxStandComp->pComponentPrivate);
	}

	omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_wfdfbdev_sink_component_Private->ports = NULL;
	omx_wfdfbdev_sink_component_Private->dec_color_mode = DEC_OUT_YUV420SEP;

  /** we could create our own port structures here
	* fixme maybe the base class could use a "port factory" function pointer?
	*/
	err = omx_base_sink_Constructor (openmaxStandComp, cComponentName);

	omx_wfdfbdev_sink_component_Private->sPortTypesParam[OMX_PortDomainVideo].nStartPortNumber = 0;
	omx_wfdfbdev_sink_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts = 1;

  /** Allocate Ports and call port constructor. */
	if ((omx_wfdfbdev_sink_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts)
		&& !omx_wfdfbdev_sink_component_Private->ports)
	{
		omx_wfdfbdev_sink_component_Private->ports =
		TCC_calloc ((omx_wfdfbdev_sink_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts), sizeof (omx_base_PortType *));
		if (!omx_wfdfbdev_sink_component_Private->ports)
		{
			return OMX_ErrorInsufficientResources;
		}
		omx_wfdfbdev_sink_component_Private->ports[0] = TCC_calloc (1, sizeof (omx_wfdfbdev_sink_component_PortType));
		if (!omx_wfdfbdev_sink_component_Private->ports[0])
		{
			return OMX_ErrorInsufficientResources;
		}
		base_video_port_Constructor (openmaxStandComp, &omx_wfdfbdev_sink_component_Private->ports[0], 0, OMX_TRUE);
	}

	pPort =(omx_wfdfbdev_sink_component_PortType *) omx_wfdfbdev_sink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];

	/** Domain specific section for the allocated port. */
	pPort->sPortParam.format.video.nFrameWidth = 0;	//DEFAULT_WIDTH;
	pPort->sPortParam.format.video.nFrameHeight = 0;	//DEFAULT_HEIGHT;
	pPort->sPortParam.format.video.nBitrate = 0;
	pPort->sPortParam.format.video.xFramerate = 30;
	pPort->sPortParam.format.video.eColorFormat = OMX_COLOR_FormatYUV420Planar;	//OMX_COLOR_Format24bitRGB888;

	//	Figure out stride, slice height, min buffer size
	pPort->sPortParam.format.video.nStride = fb_calcStride(pPort->sPortParam.format.video.nFrameWidth, 
														   pPort->sPortParam.format.video.eColorFormat);
	pPort->sPortParam.format.video.nSliceHeight = pPort->sPortParam.format.video.nFrameHeight;	//	No support for slices yet
	pPort->sPortParam.nBufferSize = VIDEO_DEC_OUT_BUFFER_SIZE; //	(OMX_U32) abs (pPort->sPortParam.format.video.nStride) * pPort->sPortParam.format.video.nSliceHeight;
	pPort->sPortParam.nBufferCountMin = 4;
	pPort->sPortParam.nBufferCountActual = 4;
	pPort->sVideoParam.eColorFormat = OMX_COLOR_FormatYUV420Planar;	//OMX_COLOR_Format24bitRGB888;
	pPort->sVideoParam.xFramerate = 30;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "In %s, bSize=%d stride=%d\n", __func__, (int) pPort->sPortParam.nBufferSize,
		   (int) pPort->sPortParam.format.video.nStride);

  /** Set configs */
	setHeader (&pPort->omxConfigCrop, sizeof (OMX_CONFIG_RECTTYPE));
	pPort->omxConfigCrop.nPortIndex = OMX_BASE_SINK_INPUTPORT_INDEX;
	pPort->omxConfigCrop.nLeft = pPort->omxConfigCrop.nTop = 0;
	pPort->omxConfigCrop.nWidth = pPort->omxConfigCrop.nHeight = 0;

	setHeader (&pPort->omxConfigRotate, sizeof (OMX_CONFIG_ROTATIONTYPE));
	pPort->omxConfigRotate.nPortIndex = OMX_BASE_SINK_INPUTPORT_INDEX;
	pPort->omxConfigRotate.nRotation = 0;	//Default: No rotation (0 degrees)

	setHeader (&pPort->omxConfigMirror, sizeof (OMX_CONFIG_MIRRORTYPE));
	pPort->omxConfigMirror.nPortIndex = OMX_BASE_SINK_INPUTPORT_INDEX;
	pPort->omxConfigMirror.eMirror = OMX_MirrorNone;	//Default: No mirroring

	setHeader (&pPort->omxConfigScale, sizeof (OMX_CONFIG_SCALEFACTORTYPE));
	pPort->omxConfigScale.nPortIndex = OMX_BASE_SINK_INPUTPORT_INDEX;
	pPort->omxConfigScale.xWidth = pPort->omxConfigScale.xHeight = 0x10000;	//Default: No scaling (scale factor = 1)

	setHeader (&pPort->omxConfigOutputPosition, sizeof (OMX_CONFIG_POINTTYPE));
	pPort->omxConfigOutputPosition.nPortIndex = OMX_BASE_SINK_INPUTPORT_INDEX;
	pPort->omxConfigOutputPosition.nX = pPort->omxConfigOutputPosition.nY = 0;	//Default: No shift in output position (0,0)

  /** set the function pointers */
	omx_wfdfbdev_sink_component_Private->destructor = omx_wfdfbdev_sink_component_Destructor;
	omx_wfdfbdev_sink_component_Private->BufferMgmtCallback = omx_wfdfbdev_sink_component_BufferMgmtCallback;
	openmaxStandComp->SetParameter = omx_wfdfbdev_sink_component_SetParameter;
	openmaxStandComp->GetParameter = omx_wfdfbdev_sink_component_GetParameter;
	openmaxStandComp->SetConfig = omx_wfdfbdev_sink_component_SetConfig;
	openmaxStandComp->GetConfig = omx_wfdfbdev_sink_component_GetConfig;

	omx_wfdfbdev_sink_component_Private->messageHandler = omx_wfdfbdev_sink_component_MessageHandler;

  /* Write in the default paramenters */
	omx_wfdfbdev_sink_component_Private->compReady = OMX_FALSE;
	if(!omx_wfdfbdev_sink_component_Private->compSyncSem)
	{
		omx_wfdfbdev_sink_component_Private->compSyncSem = TCC_calloc(1,sizeof(tsem_t));
		if(omx_wfdfbdev_sink_component_Private->compSyncSem == NULL) return OMX_ErrorInsufficientResources;
			tsem_init(omx_wfdfbdev_sink_component_Private->compSyncSem, 0);
	}

	pthread_mutex_init (&omx_wfdfbdev_sink_component_Private->fb_stop_mutex, NULL);

	omx_wfdfbdev_sink_component_Private->eState = OMX_TIME_ClockStateStopped;
	omx_wfdfbdev_sink_component_Private->xScale = 1 << 16;

	omx_wfdfbdev_sink_component_Private->fbscreenmode = FBSINK_MODE_FULL;
	omx_wfdfbdev_sink_component_Private->v_offset = 0;
	omx_wfdfbdev_sink_component_Private->h_offset = 0;
	omx_wfdfbdev_sink_component_Private->fbscanmode = FBSINK_PROGRESSIVE_SCAN;
	omx_wfdfbdev_sink_component_Private->is_fbinit = 0;
	omx_wfdfbdev_sink_component_Private->fast_stop = OMX_FALSE;
	omx_wfdfbdev_sink_component_Private->overlay_used = 0;
	omx_wfdfbdev_sink_component_Private->overlay_fd = NULL;
	omx_wfdfbdev_sink_component_Private->cable_width = 0;
	omx_wfdfbdev_sink_component_Private->cable_height = 0;
	omx_wfdfbdev_sink_component_Private->First_Video_arrive_time = 0;
	omx_wfdfbdev_sink_component_Private->Use_Buffering_Mode = OMX_TRUE;
	omx_wfdfbdev_sink_component_Private->output_type = WFD_DISPLAY_HDMI;

	return err;
}

/** The Destructor
 */
OMX_ERRORTYPE omx_wfdfbdev_sink_component_Destructor (OMX_COMPONENTTYPE * openmaxStandComp)
{
	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;
	OMX_U32   i;

	if(omx_wfdfbdev_sink_component_Private->compSyncSem) {
		tsem_deinit(omx_wfdfbdev_sink_component_Private->compSyncSem);
		TCC_free(omx_wfdfbdev_sink_component_Private->compSyncSem);
		omx_wfdfbdev_sink_component_Private->compSyncSem=NULL;
	}

	pthread_mutex_destroy (&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);

	/* frees port/s */
	if (omx_wfdfbdev_sink_component_Private->ports)
	{
		for (i = 0; i < omx_wfdfbdev_sink_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts; i++)
		{
			if (omx_wfdfbdev_sink_component_Private->ports[i])
				omx_wfdfbdev_sink_component_Private->ports[i]->PortDestructor (omx_wfdfbdev_sink_component_Private->ports[i]);
		}
		TCC_free (omx_wfdfbdev_sink_component_Private->ports);
		omx_wfdfbdev_sink_component_Private->ports = NULL;
	}

	omx_base_sink_Destructor (openmaxStandComp);
	return OMX_ErrorNone;
}


/** The initialization function
  * This function opens the frame buffer device and allocates memory for display
  * also it finds the frame buffer supported display formats
  */

unsigned long long GetTime_MSec1(void) // milli second
{
	struct timeval now;
	gettimeofday (&now, NULL);
	return ((unsigned long long) now.tv_sec) * 1000 + ((unsigned long long) now.tv_usec) / 1000;
}

OMX_ERRORTYPE omx_wfdfbdev_sink_component_Initialize (OMX_COMPONENTTYPE * openmaxStandComp)
{
	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_wfdfbdev_sink_component_PortType *pPort = (omx_wfdfbdev_sink_component_PortType *) omx_wfdfbdev_sink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];
	unsigned long framebuffer_width;
	unsigned long framebuffer_height;
	
	unsigned int uiOutputMode;
	int ii;
	int 	  fd, ret, mem;
	unsigned long long measure;
	unsigned long long walltime, newtime;
	struct timeval						tv;

	overlay_chroma_t chroma;
	unsigned int overlay_virtual_addr, g2d_virtual_addr, scaler_virtual_addr;
	
	int address[3];
	struct fb_var_screeninfo info;
#if 0
	memset(fb_window_pre,0,sizeof(unsigned int)*4);
	memset(fb_addroffset_pre,0,sizeof(unsigned int)*2);

	//Scaler --- drivers
	omx_wfdfbdev_sink_component_Private->scaler0_fd = tcc_scaler_open(TCC_SCALER0_DEV_NAME);
	if (omx_wfdfbdev_sink_component_Private->scaler0_fd < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "can't open %s\n", TCC_SCALER0_DEV_NAME);
		return OMX_ErrorBadParameter;
	}
	else
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"%s open succeed\n", TCC_SCALER0_DEV_NAME);

	//Overlay --- drivers
	omx_wfdfbdev_sink_component_Private->overlay_fd = tcc_overlay_open(TCC_OVERLAY0_DEV_NAME);
	if (omx_wfdfbdev_sink_component_Private->overlay_fd < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "can't open %s\n", TCC_OVERLAY0_DEV_NAME);
		return OMX_ErrorBadParameter;
	}
	else
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"%s open succeed\n", TCC_OVERLAY0_DEV_NAME);
	}
	//Memory --- drivers
	omx_wfdfbdev_sink_component_Private->memory_fd = open(TCC_MEMORY_DEV_NAME, O_RDWR | O_NDELAY);
	if (omx_wfdfbdev_sink_component_Private->memory_fd < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "can't open %s\n", TCC_MEMORY_DEV_NAME);
		return OMX_ErrorBadParameter;
	}
	else
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"%s open succeed\n", TCC_MEMORY_DEV_NAME);
	}
#endif
	//fb0 --- drivers
	omx_wfdfbdev_sink_component_Private->fb0_fd = tcc_fb_open(TCC_FB0_DEV_NAME);
	if (omx_wfdfbdev_sink_component_Private->fb0_fd < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "can't open %s", TCC_FB0_DEV_NAME);
		return OMX_ErrorBadParameter;
	}
	else
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"%s open succeed\n", TCC_FB0_DEV_NAME);
	}


	if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_COMPOSITE)
	{
		//Scaler --- drivers
		omx_wfdfbdev_sink_component_Private->scaler0_fd = tcc_scaler_open(TCC_SCALER0_DEV_NAME);
		if (omx_wfdfbdev_sink_component_Private->scaler0_fd < 0)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "can't open %s\n", TCC_SCALER0_DEV_NAME);
			return OMX_ErrorBadParameter;
		}
		else
		{
			printf("Pass :	Scaler Open... \n");
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"%s open succeed\n", TCC_SCALER0_DEV_NAME);
		}

		//Memory --- drivers
		omx_wfdfbdev_sink_component_Private->memory_fd = open(TCC_MEMORY_DEV_NAME, O_RDWR | O_NDELAY);
		if (omx_wfdfbdev_sink_component_Private->memory_fd < 0)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "can't open %s\n", TCC_MEMORY_DEV_NAME);
			return OMX_ErrorBadParameter;
		}
		else
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"%s open succeed\n", TCC_MEMORY_DEV_NAME);
		}

		//overlay buffer
		pmap_get_info(TCC_PMAP_OVERLAY, &omx_wfdfbdev_sink_component_Private->pmap_overlay);
		omx_wfdfbdev_sink_component_Private->overlay_map_ptr = (unsigned char*)mmap(0, 
																					omx_wfdfbdev_sink_component_Private->pmap_overlay.size, 
																					PROT_READ | PROT_WRITE, MAP_SHARED, 
																					omx_wfdfbdev_sink_component_Private->memory_fd, 
																					omx_wfdfbdev_sink_component_Private->pmap_overlay.base);
		if(omx_wfdfbdev_sink_component_Private->overlay_map_ptr == MAP_FAILED)
		{
			printf("Failed :  Overlay Pmap... \n");
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "mmap failed. fd(%d), addr(0x%x), size(%d)\n", 
															   omx_wfdfbdev_sink_component_Private->memory_fd, 
															   omx_wfdfbdev_sink_component_Private->pmap_overlay.base, 
															   omx_wfdfbdev_sink_component_Private->pmap_overlay.size);
			return OMX_ErrorBadParameter;
		}
		printf("Pass :	Overlay Pmap... \n");
		overlay_virtual_addr = (unsigned int)omx_wfdfbdev_sink_component_Private->overlay_map_ptr;
		memset(overlay_virtual_addr, 0x00, omx_wfdfbdev_sink_component_Private->pmap_overlay.size);

		//frame buffer setting
		glcd_double_buf_index = 0;
		glcd_double_buf_num = NUMBER_OF_BUFFER;
		for(ii=0; ii<glcd_double_buf_num; ii++)
		{
			glcd_double_buf_address[ii] = omx_wfdfbdev_sink_component_Private->pmap_overlay.base + (1280 * 720 *  2) * ii;
		}
	}
	
#if 0
	if((omx_wfdfbdev_sink_component_Private->output_type == LINUX_SNOR_DONGLE) || (omx_wfdfbdev_sink_component_Private->output_type == LINUX_PLATFORM))
	{
		//overlay buffer
		pmap_get_info(TCC_PMAP_OVERLAY, &omx_wfdfbdev_sink_component_Private->pmap_overlay);
		omx_wfdfbdev_sink_component_Private->overlay_map_ptr = (unsigned char*)mmap(0, omx_wfdfbdev_sink_component_Private->pmap_overlay.size, PROT_READ | PROT_WRITE, MAP_SHARED, omx_wfdfbdev_sink_component_Private->memory_fd, omx_wfdfbdev_sink_component_Private->pmap_overlay.base);
		if(omx_wfdfbdev_sink_component_Private->overlay_map_ptr == MAP_FAILED)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "mmap failed. fd(%d), addr(0x%x), size(%d)\n", omx_wfdfbdev_sink_component_Private->memory_fd, omx_wfdfbdev_sink_component_Private->pmap_overlay.base, omx_wfdfbdev_sink_component_Private->pmap_overlay.size);
			return OMX_ErrorBadParameter;
		}
		overlay_virtual_addr = (unsigned int)omx_wfdfbdev_sink_component_Private->overlay_map_ptr;
		memset(overlay_virtual_addr, 0x00, omx_wfdfbdev_sink_component_Private->pmap_overlay.size);
	}
  
	if(tcc_overlay_get_screen_info(omx_wfdfbdev_sink_component_Private->overlay_fd,&omx_wfdfbdev_sink_component_Private->overlay_info) != TCC_OVERLAY_ERROR_NONE)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "overlay_get_info error\n");
	}

	if((omx_wfdfbdev_sink_component_Private->overlay_info.width * omx_wfdfbdev_sink_component_Private->overlay_info.height) > (TCC_FB_EXTEND_DISPLAY_WIDTH * TCC_FB_EXTEND_DISPLAY_HEIGHT))
	{
		framebuffer_width = omx_wfdfbdev_sink_component_Private->overlay_info.width;
		framebuffer_height = omx_wfdfbdev_sink_component_Private->overlay_info.height;
	}
	else
	{
		framebuffer_width = TCC_FB_EXTEND_DISPLAY_WIDTH;
		framebuffer_height = TCC_FB_EXTEND_DISPLAY_HEIGHT;
	}
#endif	
#if 0
	if((omx_wfdfbdev_sink_component_Private->output_type == LINUX_SNOR_DONGLE) || (omx_wfdfbdev_sink_component_Private->output_type == LINUX_PLATFORM))
	{
		//frame buffer setting
		glcd_double_buf_index = 0;
		glcd_double_buf_num = NUMBER_OF_BUFFER;
		for(ii=0; ii<glcd_double_buf_num; ii++)
		{
			glcd_double_buf_address[ii] = omx_wfdfbdev_sink_component_Private->pmap_overlay.base + (framebuffer_width * framebuffer_height *  2) * ii;
		}
	}
	tcc_fb_get_vscreen_info(omx_wfdfbdev_sink_component_Private->fb0_fd,&info);
#endif	  
	omx_wfdfbdev_sink_component_Private->fbstartX = 0;
	omx_wfdfbdev_sink_component_Private->fbstartY = 0;
	omx_wfdfbdev_sink_component_Private->fbwidth = omx_wfdfbdev_sink_component_Private->cable_width;
	omx_wfdfbdev_sink_component_Private->fbheight = omx_wfdfbdev_sink_component_Private->cable_height;
#if 0	
	else if(omx_wfdfbdev_sink_component_Private->output_type == LINUX_CCM)
	{
		omx_wfdfbdev_sink_component_Private->fbstartX = omx_wfdfbdev_sink_component_Private->cable_posx;
		omx_wfdfbdev_sink_component_Private->fbstartY = omx_wfdfbdev_sink_component_Private->cable_posy;
		omx_wfdfbdev_sink_component_Private->fbwidth =	omx_wfdfbdev_sink_component_Private->cable_width;
		omx_wfdfbdev_sink_component_Private->fbheight = omx_wfdfbdev_sink_component_Private->cable_height;
	}
	
	else
	{
		;
	}
#endif	
#if 0
	if((omx_wfdfbdev_sink_component_Private->output_type == LINUX_SNOR_DONGLE) || (omx_wfdfbdev_sink_component_Private->output_type == LINUX_PLATFORM))
	{
		//overlay setting
		ret =tcc_overlay_set_screen_info(omx_wfdfbdev_sink_component_Private->overlay_fd,&omx_wfdfbdev_sink_component_Private->overlay_info);

		if(ret < 0)
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "overlay 0 configure error\n");
	}

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "mmap framebuffer memory =%x omx_wfdfbdev_sink_component_Private->product=%d stride=%d\n",
															   (int) omx_wfdfbdev_sink_component_Private->scr_ptr, 
															   (int) omx_wfdfbdev_sink_component_Private->product,
															   (int) omx_wfdfbdev_sink_component_Private->fbstride);

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Successfully opened %s for display.\n", "/dev/fb0");

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Display Size: %u x %u\n", 
																(int) omx_wfdfbdev_sink_component_Private->fbwidth,
																(int) omx_wfdfbdev_sink_component_Private->fbheight);

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Bitdepth: %u\n", 
																(int) omx_wfdfbdev_sink_component_Private->fbbpp);
#endif
	omx_wfdfbdev_sink_component_Private->compReady = OMX_TRUE;
	omx_wfdfbdev_sink_component_Private->overlay_setting_done = 0;
	omx_wfdfbdev_sink_component_Private->LastAudioTime = 0;
	/*Indicate that comp. is ready*/
	tsem_up(omx_wfdfbdev_sink_component_Private->compSyncSem);
	return OMX_ErrorNone;
}

/** The deinitialization function
  * It deallocates the frame buffer memory, and closes frame buffer
  */
OMX_ERRORTYPE omx_wfdfbdev_sink_component_Deinit (OMX_COMPONENTTYPE * openmaxStandComp)
{
	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;
	int 	  fd, ret;
	unsigned int fb_window[4];

#if 0
	// hdmi  ȭ  ڵ Դϴ.
	if(omx_wfdfbdev_sink_component_Private->overlay_fd == NULL)
		omx_wfdfbdev_sink_component_Private->overlay_fd = tcc_overlay_open(TCC_OVERLAY0_DEV_NAME);

	if(omx_wfdfbdev_sink_component_Private->overlay_fd != NULL)
	{
		if(tcc_overlay_disable(omx_wfdfbdev_sink_component_Private->overlay_fd) < 0)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "OVERLAY_SET_DISABLE ERROR!!!!!!!!\n");
		}
		omx_wfdfbdev_sink_component_Private->overlay_used = 0;
		tcc_overlay_close(omx_wfdfbdev_sink_component_Private->overlay_fd);
		omx_wfdfbdev_sink_component_Private->overlay_fd = NULL;
	}
#endif
#if 0
	if(omx_wfdfbdev_sink_component_Private->fb0_fd)
	{
		if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_HDMI)
		{
			tcc_lcdc_image_update lcdc_display;
			memset(&lcdc_display, 0, sizeof(tcc_lcdc_image_update));
			lcdc_display.Lcdc_layer = LCDC_LAYER_2;
			lcdc_display.enable 	= 0;
			ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_LCDC_HDMI_DISPLAY, &lcdc_display);
		}
		else if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_LCD)
		{
			tcc_lcdc_image_update lcdc_display;
			memset(&lcdc_display, 0, sizeof(tcc_lcdc_image_update));
			lcdc_display.Lcdc_layer = LCDC_LAYER_2;
			lcdc_display.enable 	= 0;
			ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_LCDC_DISPLAY_UPDATE, &lcdc_display);
		}
		tcc_fb_close(omx_wfdfbdev_sink_component_Private->fb0_fd);
		omx_wfdfbdev_sink_component_Private->fb0_fd = NULL;
	}
#endif
#if 0
	tcc_scaler_close(omx_wfdfbdev_sink_component_Private->scaler0_fd);
	close(omx_wfdfbdev_sink_component_Private->memory_fd);
#endif
	omx_wfdfbdev_sink_component_Private->LastAudioTime = 0;
	omx_wfdfbdev_sink_component_Private->compReady = OMX_FALSE;
	tsem_reset(omx_wfdfbdev_sink_component_Private->compSyncSem);

	return OMX_ErrorNone;
}

int omx_wfdfbdev_sink_component_DisplayFrame(omx_wfdfbdev_sink_component_PrivateType *
														   omx_wfdfbdev_sink_component_Private,
														   OMX_BUFFERHEADERTYPE * pInputBuffer)
{
	omx_wfdfbdev_sink_component_PortType *pPort =
		(omx_wfdfbdev_sink_component_PortType *) omx_wfdfbdev_sink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];

	OMX_U8 *input_src_ptr = (OMX_U8 *) (pInputBuffer->pBuffer);

	OMX_U32 input_src_width, input_src_height;
	OMX_U32 input_src_crop_top, input_src_crop_bottom, input_src_crop_left, input_src_crop_right;


	OMX_U32 frame_buffer;

	OMX_U32 input_dest_x;
	OMX_U32 input_dest_y;
	OMX_U32 input_dest_width;
	OMX_U32 input_dest_height;

	OMX_U32 temp_width;
	OMX_U32 temp_height;

	OMX_U32 out_w, out_h, out_w_offset, out_h_offset;

	int address[15];
	int ret;

	tcc_scaler_info_t scaler_src;
	tcc_scaler_info_t scaler_dst;

	image_info_t overlay_src;

	if(omx_wfdfbdev_sink_component_Private->fb0_fd == NULL)
	{
		return 0;
	}
	int width, height;
    OMX_VIDEO_PORTDEFINITIONTYPE *decout_frame_info = pInputBuffer->pInputPortPrivate;
	input_src_width  = decout_frame_info->nFrameWidth;//pPort->sPortParam.format.video.nFrameWidth;// - pPort->sPortParam.format.video.VideoCropInfo.nRight;
	input_src_height = decout_frame_info->nFrameHeight;//pPort->sPortParam.format.video.nFrameHeight;// - pPort->sPortParam.format.video.VideoCropInfo.nBottom;
    input_src_crop_top = decout_frame_info->VideoCropInfo.nTop;
    input_src_crop_bottom = decout_frame_info->VideoCropInfo.nBottom;
    input_src_crop_left = decout_frame_info->VideoCropInfo.nLeft;
    input_src_crop_right = decout_frame_info->VideoCropInfo.nRight;
 
 	/* vpu output physical address */
	address[0] = (*(input_src_ptr+3)<<24) | (*(input_src_ptr+2)<<16) | (*(input_src_ptr+1)<<8) | *(input_src_ptr) ;	input_src_ptr+=4;
	address[1] = (*(input_src_ptr+3)<<24) | (*(input_src_ptr+2)<<16) | (*(input_src_ptr+1)<<8) | *(input_src_ptr) ;	input_src_ptr+=4;
	address[2] = (*(input_src_ptr+3)<<24) | (*(input_src_ptr+2)<<16) | (*(input_src_ptr+1)<<8) | *(input_src_ptr) ;	input_src_ptr+=4;


	input_dest_x = omx_wfdfbdev_sink_component_Private->fbstartX;
	input_dest_y = omx_wfdfbdev_sink_component_Private->fbstartY;
	
	input_dest_width = omx_wfdfbdev_sink_component_Private->fbwidth;
	input_dest_height = omx_wfdfbdev_sink_component_Private->fbheight; 

	TCC_Aspect_Ratio_Calculate_Image (input_src_width, 
										input_src_height, 
										input_dest_width, 
										input_dest_height,
										&out_w, 
										&out_h, 
										&out_w_offset, 
										&out_h_offset,
										omx_wfdfbdev_sink_component_Private->fbscreenmode);


	if((omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_LCD) || (omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_HDMI))
	{
		tcc_lcdc_image_update lcdc_display;
		memset(&lcdc_display, 0, sizeof(tcc_lcdc_image_update));
		lcdc_display.Lcdc_layer = LCDC_LAYER_MAX; //	TCC8925
		lcdc_display.enable 	= 1;
		lcdc_display.addr0		= address[0]; //Y
		lcdc_display.addr1		= address[1]; //U
		lcdc_display.addr2		= address[2]; //V
	
		lcdc_display.offset_x = input_dest_x + out_w_offset;
		lcdc_display.offset_y = input_dest_y + out_h_offset;
	
		lcdc_display.Frame_width	= ((input_src_width + 15) >> 4) << 4;
		lcdc_display.Frame_height	= input_src_height;
		
		lcdc_display.Image_width	= out_w;
		lcdc_display.Image_height	= out_h;
		
		lcdc_display.on_the_fly = 1;
		lcdc_display.fmt = TCC_LCDC_IMG_FMT_YUV420ITL0;
		lcdc_display.crop_top = 0;
		lcdc_display.crop_bottom = lcdc_display.Frame_height - input_src_crop_bottom;
		lcdc_display.crop_left = 0;
		lcdc_display.crop_right = lcdc_display.Frame_width - input_src_crop_right; 
        
		omx_wfdfbdev_sink_component_Private->LastVideoDisplayTime = GetTime_MSec1();

		if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_HDMI)
		{
			ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_LCDC_HDMI_DISPLAY, &lcdc_display);
		}
		else if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_LCD)
		{
			ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_LCDC_DISPLAY_UPDATE, &lcdc_display);
		}
#if 0
		else if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_COMPOSITE)
		{
			ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_COMPOSITE_IOCTL_UPDATE, &lcdc_display);
		}
#endif
		else
			;
		
	}
	else if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_COMPOSITE) /*For LitBig, FullSeg BSP, It must be modified*/
	{
		 /* get framebuffer address  */
		frame_buffer = get_framebuffer_address();
		/* congifuration zoom*/
		scaler_src.w_x = 0;
		scaler_src.w_y = 0;
		scaler_src.w_w = input_src_width;
		scaler_src.w_h = input_src_height;

		scaler_src.addr_y = (char*)address[0];
		scaler_src.addr_u = (char*)address[1];
		scaler_src.addr_v = (char*)address[2];

		scaler_src.x = 0;
		scaler_src.y = 0;
		scaler_src.w = input_src_width;
		scaler_src.h = input_src_height;

		if(VIDEO_DECODER_CHROMA_INTERLEAVE)
			scaler_src.format = SCALER_YUV420_inter;
		else
			scaler_src.format = SCALER_YUV420_sp;			


		scaler_dst.addr_y = (char*)frame_buffer;
		scaler_dst.addr_u = NULL;
		scaler_dst.addr_v = NULL;

		scaler_dst.x = 0;
		scaler_dst.y = 0;
		scaler_dst.w_x = 0;
		scaler_dst.w_y = 0;
		scaler_dst.w = out_w;
		scaler_dst.h = out_h;
		scaler_dst.w_w = out_w;
		scaler_dst.w_h = out_h;
		scaler_dst.format = SCALER_YUV422_sq0;
		
		if(tcc_scaler_execute(omx_wfdfbdev_sink_component_Private->scaler0_fd, &scaler_src, &scaler_dst) < 0)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[%d][%s] scaler error\n", __LINE__, __FUNCTION__);
		}

		struct tcc_lcdc_image_update lcdc_display;
		memset(&lcdc_display, 0, sizeof(tcc_lcdc_image_update));


		lcdc_display.Lcdc_layer = LCDC_LAYER_MAX;
		lcdc_display.enable 	= 1;
		lcdc_display.addr0		= frame_buffer;
		lcdc_display.addr1		= lcdc_display.addr0+(out_w*out_h);
		lcdc_display.addr2		= lcdc_display.addr1+(out_w*out_h/4);
	
		lcdc_display.offset_x = input_dest_x + out_w_offset;
		lcdc_display.offset_y = input_dest_y + out_h_offset;
	
		lcdc_display.Frame_width  = ((out_w + 15) >> 4) << 4;
		lcdc_display.Frame_height = out_h;
	
		lcdc_display.Image_width	= out_w;
		lcdc_display.Image_height	= out_h;
	
		lcdc_display.fmt = TCC_LCDC_IMG_FMT_YUV422SP;
		lcdc_display.crop_top = 0;
		lcdc_display.crop_bottom = lcdc_display.Frame_height;
		lcdc_display.crop_left = 0;
		lcdc_display.crop_right = lcdc_display.Frame_width; 
	
		lcdc_display.outputMode = OUTPUT_COMPOSITE;
		lcdc_display.output_path = 0;
		lcdc_display.on_the_fly = 0;
		
		ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_COMPOSITE_IOCTL_UPDATE, &lcdc_display);
	}
	else
	{
	 ;
	}
	return 0;
}

/** buffer management callback function
  * takes one input buffer and displays its contents
  */
int accurate_sleep(unsigned long long msec)
{
	unsigned long long start = GetTime_MSec1();

	if(msec > 1000)
		return;
		
	while(1)
	{
		if(msec < (GetTime_MSec1() - start))
			return;

		usleep(1000);
	}

}

void omx_wfdfbdev_sink_component_BufferMgmtCallback (OMX_COMPONENTTYPE * openmaxStandComp,
												  OMX_BUFFERHEADERTYPE * pInputBuffer)
{
	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;

	OMX_U8	 *input_src_ptr = (OMX_U8 *) (pInputBuffer->pBuffer);

	/**  FIXME: Configuration values should be clamped to prevent memory trampling and potential segfaults.
	*  It might be best to store clamped AND unclamped values on a per-port basis so that OMX_GetConfig
	*  can still return the unclamped ones.
	*/

	omx_base_clock_PortType *pClockPort;
	OMX_HANDLETYPE hclkComponent;
	OMX_TIME_CONFIG_MEDIATIMEREQUESTSPEEDTYPE sMediaTimeNotify;
	OMX_BOOL SendFrame = TRUE;
	OMX_TICKS  AudioCurrentTime = 0;
	OMX_TICKS  System_current_clocks = 0;
	OMX_BOOL Skip_Display = OMX_FALSE;
	int address[15];
	pthread_mutex_lock(&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);

	/* don't move this routine!! this routine should be done first!! */
	if (omx_wfdfbdev_sink_component_Private->compReady == OMX_FALSE) {
		if(omx_wfdfbdev_sink_component_Private->state == OMX_StateExecuting) {
			/*wait for avformat to be ready*/
			tsem_down(omx_wfdfbdev_sink_component_Private->compSyncSem);
		} else {
			pthread_mutex_unlock(&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);
			return;
		}
	}
	if(omx_wfdfbdev_sink_component_Private->fast_stop)
	{
		pInputBuffer->nFilledLen = 0;
		usleep(1000);
		pthread_mutex_unlock(&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);
		return;
	}

	if(pInputBuffer->nFlags & OMX_BUFFERFLAG_EOS)
	{
		pInputBuffer->nFilledLen = 0;
		usleep(1000);
		pthread_mutex_unlock(&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);
		return;
	}

	if(omx_wfdfbdev_sink_component_Private->Use_Buffering_Mode == OMX_TRUE)
	{
		//Get Current Audio Time-stamp!!
		(*(omx_wfdfbdev_sink_component_Private->callbacks->EventHandler))  (openmaxStandComp, 
																			NULL,
																			OMX_EventVendorGetAudioTimeStamp,
																			NULL, /*Skip Start*/
																			NULL,
																			(void*)&AudioCurrentTime);



		System_current_clocks = GetTime_MSec1(); // get current system clocks...
		OMX_TICKS  current_clock = ((System_current_clocks - omx_wfdfbdev_sink_component_Private->Syatem_base_clocks)*1000) + omx_wfdfbdev_sink_component_Private->Media_base_clocks - 50*1000;
		OMX_TICKS  media_clock = pInputBuffer->nTimeStamp;	//video pts
#if 0
		printf("system clock %lld  video pts %lld audio pts %lld  A/V diff %lld  S/A diff %lld\n",current_clock,
																								  media_clock,
																								  AudioCurrentTime,
																								  media_clock - AudioCurrentTime,
																								  current_clock - AudioCurrentTime);
#endif
		if(((AudioCurrentTime - current_clock) > 60*1000)|| ((current_clock - AudioCurrentTime) > 60*1000))
		{
			(*(omx_wfdfbdev_sink_component_Private->callbacks->EventHandler))  (openmaxStandComp, 
																				NULL,
																				OMX_EventVendorResetBaseClocks,
																				NULL, /*Skip Start*/
																				NULL,
																				NULL);
		}
		else 
		{
			if(10*1000 < (media_clock - current_clock))
			{
				// Video is early... wait for display...
				accurate_sleep((media_clock - current_clock)/1000);
			}
		}
		
		System_current_clocks = GetTime_MSec1();
		if((System_current_clocks - omx_wfdfbdev_sink_component_Private->LastVideoDisplayTime > 35)||(System_current_clocks - omx_wfdfbdev_sink_component_Private->LastVideoDisplayTime < 30))
		{
			//printf("Display Interval %lld \n",System_current_clocks - omx_wfdfbdev_sink_component_Private->LastVideoDisplayTime );
		}
	}

	if(omx_wfdfbdev_sink_component_DisplayFrame(omx_wfdfbdev_sink_component_Private, pInputBuffer) < 0)
	{
		pthread_mutex_unlock(&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);
		return;
	}

	/* Display   Frame .*/
	pInputBuffer->nFilledLen = 0;
	pInputBuffer->nFlags = 0;
	pthread_mutex_unlock(&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);
}

OMX_ERRORTYPE omx_wfdfbdev_sink_component_SetConfig (OMX_IN OMX_HANDLETYPE hComponent,
												  OMX_IN OMX_INDEXTYPE nIndex, OMX_IN OMX_PTR pComponentConfigStructure)
{

	OMX_ERRORTYPE err = OMX_ErrorNone;

	/* Check which structure we are being fed and make control its header */
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *) hComponent;

	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_wfdfbdev_sink_component_PortType *pPort;

	TCC_VIDEO_SET_SCREENMODE_t *pFB_DispMode= pComponentConfigStructure;
	TCC_VIDEO_SET_RESOLUTION_t	*pFB_InputResolution = pComponentConfigStructure;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Setting parameter %i\n", nIndex);

	switch (nIndex)
	{
		case OMX_IndexVendorConfigExtendDisplayModeSet:
		{
			OUTPUT_INFO *info;

			info = (OUTPUT_INFO*)pComponentConfigStructure;
			if(info->mode == WFD_DISPLAY_HDMI)
			{
				omx_wfdfbdev_sink_component_Private->output_type = WFD_DISPLAY_HDMI;
				omx_wfdfbdev_sink_component_Private->cable_mode = OUTPUT_HDMI;
				omx_wfdfbdev_sink_component_Private->cable_width = info->width;
				omx_wfdfbdev_sink_component_Private->cable_height = info->height;
				omx_wfdfbdev_sink_component_Private->cable_posx = info->pos_x;
				omx_wfdfbdev_sink_component_Private->cable_posy = info->pos_y;
				printf("[FB_SINK]HDMI OUT X:%d	Y:%d  W:%d	H:%d \n",omx_wfdfbdev_sink_component_Private->cable_posx,
																omx_wfdfbdev_sink_component_Private->cable_posy,
																omx_wfdfbdev_sink_component_Private->cable_width,
																omx_wfdfbdev_sink_component_Private->cable_height);
			}
			else if(info->mode == WFD_DISPLAY_LCD)
			{
				omx_wfdfbdev_sink_component_Private->output_type = WFD_DISPLAY_LCD;
				omx_wfdfbdev_sink_component_Private->cable_width = info->width;
				omx_wfdfbdev_sink_component_Private->cable_height = info->height;
				omx_wfdfbdev_sink_component_Private->cable_posx = info->pos_x;
				omx_wfdfbdev_sink_component_Private->cable_posy = info->pos_y;
				printf("[FB_SINK]LCD OUT X:%d  Y:%d  W:%d  H:%d \n",omx_wfdfbdev_sink_component_Private->cable_posx,
																omx_wfdfbdev_sink_component_Private->cable_posy,
																omx_wfdfbdev_sink_component_Private->cable_width,
																omx_wfdfbdev_sink_component_Private->cable_height);
			}
			else if(info->mode == WFD_DISPLAY_COMPOSITE)
			{		   
				omx_wfdfbdev_sink_component_Private->output_type = WFD_DISPLAY_COMPOSITE;
				omx_wfdfbdev_sink_component_Private->cable_mode = WFD_DISPLAY_COMPOSITE;
				omx_wfdfbdev_sink_component_Private->cable_width = info->width;
				omx_wfdfbdev_sink_component_Private->cable_height = info->height;
				omx_wfdfbdev_sink_component_Private->cable_posx = info->pos_x;
				omx_wfdfbdev_sink_component_Private->cable_posy = info->pos_y;
				printf("[FB_SINK]COMPOSITE OUT X:%d  Y:%d  W:%d  H:%d \n",omx_wfdfbdev_sink_component_Private->cable_posx,
																omx_wfdfbdev_sink_component_Private->cable_posy,
																omx_wfdfbdev_sink_component_Private->cable_width,
																omx_wfdfbdev_sink_component_Private->cable_height);

			}
			else
				;
		}
		break;
		case OMX_IndexVendorConfigFastStop:
		{
			OMX_AUDIO_CONFIG_INFOTYPE *info;
			info = (OMX_AUDIO_CONFIG_INFOTYPE*)pComponentConfigStructure;
			omx_wfdfbdev_sink_component_Private->fast_stop = info->faststop;
		}
		break;
		
		case OMX_IndexVendorSetWFDBaseTimeReset:
		{	
			BaseClockInfo *clock_info = (BaseClockInfo*)pComponentConfigStructure;
			
			omx_wfdfbdev_sink_component_Private->Syatem_base_clocks = clock_info->system_base_clocks;
			omx_wfdfbdev_sink_component_Private->Media_base_clocks	= clock_info->media_base_clocks;
		}
		break;	

		case OMX_IndexVendorSetDisplayDisable:
		{
			pthread_mutex_lock(&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);	
			if(omx_wfdfbdev_sink_component_Private->fb0_fd)
			{
				if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_HDMI)
				{
					tcc_lcdc_image_update lcdc_display;
					memset(&lcdc_display, 0, sizeof(tcc_lcdc_image_update));
					lcdc_display.Lcdc_layer = LCDC_LAYER_MAX;
					lcdc_display.enable 	= 0;
					ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_LCDC_HDMI_DISPLAY, &lcdc_display);
				}
				else if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_LCD)
				{
					tcc_lcdc_image_update lcdc_display;
					memset(&lcdc_display, 0, sizeof(tcc_lcdc_image_update));
					lcdc_display.Lcdc_layer = LCDC_LAYER_MAX;
					lcdc_display.enable 	= 0;
					ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_LCDC_DISPLAY_UPDATE, &lcdc_display);
				}
				else if(omx_wfdfbdev_sink_component_Private->output_type == WFD_DISPLAY_COMPOSITE)
				{
					tcc_lcdc_image_update lcdc_display;
					memset(&lcdc_display, 0, sizeof(tcc_lcdc_image_update));
					lcdc_display.Lcdc_layer = LCDC_LAYER_MAX;
					lcdc_display.enable 	= 0;
					ioctl(omx_wfdfbdev_sink_component_Private->fb0_fd, TCC_COMPOSITE_IOCTL_UPDATE, &lcdc_display);
					tcc_scaler_close(omx_wfdfbdev_sink_component_Private->scaler0_fd);
				}				
				tcc_fb_close(omx_wfdfbdev_sink_component_Private->fb0_fd);
				omx_wfdfbdev_sink_component_Private->fb0_fd = NULL;
			}
			pthread_mutex_unlock(&omx_wfdfbdev_sink_component_Private->fb_stop_mutex);
		}
		break;
		
		case OMX_IndexVendorSetBufferingMode:
		{
			if(omx_wfdfbdev_sink_component_Private->Use_Buffering_Mode == OMX_FALSE)
			{
				printf("Video Sink Enable Buffering Mode................\n");
				omx_wfdfbdev_sink_component_Private->Use_Buffering_Mode = OMX_TRUE;	
			}
			else		
			{
				printf("Video Sink Disable Buffering Mode................\n");
				omx_wfdfbdev_sink_component_Private->Use_Buffering_Mode = OMX_FALSE;	
			}
		}
		break;

		default:	// delegate to superclass
			return omx_base_component_SetConfig (hComponent, nIndex, pComponentConfigStructure);
	}
	return err;
}

OMX_ERRORTYPE omx_wfdfbdev_sink_component_GetConfig (OMX_IN OMX_HANDLETYPE hComponent,
												  OMX_IN OMX_INDEXTYPE nIndex,
												  OMX_INOUT OMX_PTR pComponentConfigStructure)
{

	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *) hComponent;
	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_wfdfbdev_sink_component_PortType *pPort;

	if (pComponentConfigStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Getting configuration %i\n", nIndex);

	/* Check which structure we are being fed and fill its header */

	switch (nIndex)
	{

		default:	// delegate to superclass
			return omx_base_component_GetConfig (hComponent, nIndex, pComponentConfigStructure);
	}
	return OMX_ErrorNone;
}

OMX_ERRORTYPE omx_wfdfbdev_sink_component_SetParameter (OMX_IN OMX_HANDLETYPE hComponent,
													 OMX_IN OMX_INDEXTYPE nParamIndex,
													 OMX_IN OMX_PTR ComponentParameterStructure)
{
	OMX_ERRORTYPE err = OMX_ErrorNone;
	/* Check which structure we are being fed and make control its header */
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *) hComponent;
	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_wfdfbdev_sink_component_PortType *pPort;
	OMX_PARAM_PORTDEFINITIONTYPE *pPortDef;

	if (ComponentParameterStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Setting parameter %i\n", nParamIndex);
	switch (nParamIndex)
	{
        case OMX_IndexParamPortDefinition:
            pPortDef = (OMX_PARAM_PORTDEFINITIONTYPE *) ComponentParameterStructure;
            pPort =(omx_wfdfbdev_sink_component_PortType *) omx_wfdfbdev_sink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];

            pPort->sPortParam.format.video.nFrameWidth = pPortDef->format.video.nFrameWidth;
            pPort->sPortParam.format.video.nFrameHeight = pPortDef->format.video.nFrameHeight;

            //	Figure out stride, slice height, min buffer size
            pPort->sPortParam.format.video.nStride = fb_calcStride(pPort->sPortParam.format.video.nFrameWidth, 
            pPort->sVideoParam.eColorFormat);
            pPort->sPortParam.format.video.nSliceHeight = pPort->sPortParam.format.video.nFrameHeight;

            pPort->sPortParam.format.video.VideoCropInfo.nTop	 = pPortDef->format.video.VideoCropInfo.nTop;
            pPort->sPortParam.format.video.VideoCropInfo.nBottom = pPortDef->format.video.VideoCropInfo.nBottom;
            pPort->sPortParam.format.video.VideoCropInfo.nLeft	 = pPortDef->format.video.VideoCropInfo.nLeft;
            pPort->sPortParam.format.video.VideoCropInfo.nRight  = pPortDef->format.video.VideoCropInfo.nRight;
            TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"Set Display size:%dx%d Crop info Top:%d-Bottom:%d-Left:%d-Right:%d \n", 
                                                                                                pPort->sPortParam.format.video.nFrameWidth,
                                                                                                pPort->sPortParam.format.video.nFrameHeight,
                                                                                                pPort->sPortParam.format.video.VideoCropInfo.nTop, 
                                                                                                pPort->sPortParam.format.video.VideoCropInfo.nBottom,
                                                                                                pPort->sPortParam.format.video.VideoCropInfo.nLeft,
                                                                                                pPort->sPortParam.format.video.VideoCropInfo.nRight);
			break;
	
		default:	/*Call the base component function */
			return omx_base_component_SetParameter (hComponent, nParamIndex, ComponentParameterStructure);
	}
	return err;
}

OMX_ERRORTYPE omx_wfdfbdev_sink_component_GetParameter (OMX_IN OMX_HANDLETYPE hComponent,
													 OMX_IN OMX_INDEXTYPE nParamIndex,
													 OMX_INOUT OMX_PTR ComponentParameterStructure)
{

	OMX_VIDEO_PARAM_PORTFORMATTYPE *pVideoPortFormat;
	OMX_OTHER_PARAM_PORTFORMATTYPE *pOtherPortFormat;
	OMX_ERRORTYPE err = OMX_ErrorNone;
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *) hComponent;
	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_wfdfbdev_sink_component_PortType *pPort = (omx_wfdfbdev_sink_component_PortType *) omx_wfdfbdev_sink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];
	omx_base_clock_PortType *pClockPort = (omx_base_clock_PortType *) omx_wfdfbdev_sink_component_Private->ports[1];
	if (ComponentParameterStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Getting parameter %i\n", nParamIndex);
	/* Check which structure we are being fed and fill its header */
	switch (nParamIndex)
	{
		case OMX_IndexParamVideoInit:
			if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_PORT_PARAM_TYPE))) != OMX_ErrorNone)
			{
				break;
			}
			memcpy (ComponentParameterStructure, &omx_wfdfbdev_sink_component_Private->sPortTypesParam[OMX_PortDomainVideo],
					sizeof (OMX_PORT_PARAM_TYPE));
			break;
		case OMX_IndexParamOtherInit:
			if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_PORT_PARAM_TYPE))) != OMX_ErrorNone)
			{
				break;
			}
			memcpy (ComponentParameterStructure, &omx_wfdfbdev_sink_component_Private->sPortTypesParam[OMX_PortDomainOther],
					sizeof (OMX_PORT_PARAM_TYPE));
			break;
		case OMX_IndexParamVideoPortFormat:
			pVideoPortFormat = (OMX_VIDEO_PARAM_PORTFORMATTYPE *) ComponentParameterStructure;
			if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_VIDEO_PARAM_PORTFORMATTYPE))) != OMX_ErrorNone)
			{
				break;
			}
			if (pVideoPortFormat->nPortIndex < 1)
			{
				memcpy (pVideoPortFormat, &pPort->sVideoParam, sizeof (OMX_VIDEO_PARAM_PORTFORMATTYPE));
			}
			else
			{
				return OMX_ErrorBadPortIndex;
			}
			break;
		case OMX_IndexParamOtherPortFormat:

			pOtherPortFormat = (OMX_OTHER_PARAM_PORTFORMATTYPE *) ComponentParameterStructure;

			if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_OTHER_PARAM_PORTFORMATTYPE))) != OMX_ErrorNone)
			{
				break;
			}
			if (pOtherPortFormat->nPortIndex == 1)
			{
				memcpy (pOtherPortFormat, &pClockPort->sOtherParam, sizeof (OMX_OTHER_PARAM_PORTFORMATTYPE));
			}
			else
			{
				return OMX_ErrorBadPortIndex;
			}
			break;

		default:	/*Call the base component function */
			return omx_base_component_GetParameter (hComponent, nParamIndex, ComponentParameterStructure);
	}
	return err;
}

OMX_ERRORTYPE omx_wfdfbdev_sink_component_MessageHandler (OMX_COMPONENTTYPE * openmaxStandComp,
													   internalRequestMessageType * message)
{

	omx_wfdfbdev_sink_component_PrivateType *omx_wfdfbdev_sink_component_Private =
		(omx_wfdfbdev_sink_component_PrivateType *) openmaxStandComp->pComponentPrivate;
	OMX_ERRORTYPE err;
	OMX_STATETYPE eState;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s\n", __func__);
	eState = omx_wfdfbdev_sink_component_Private->state;	//storing current state

	if (message->messageType == OMX_CommandStateSet)
	{
		if ((message->messageParam == OMX_StateExecuting) && (omx_wfdfbdev_sink_component_Private->state == OMX_StateIdle))
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s sink component from loaded to idle \n", __func__);
			err = omx_wfdfbdev_sink_component_Initialize (openmaxStandComp);
			if (err != OMX_ErrorNone)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Video Sink Init Failed Error=%x\n", __func__, err);
				return err;
			}
			omx_wfdfbdev_sink_component_Private->is_fbinit = OMX_TRUE;
		}
		else if ((message->messageParam == OMX_StateLoaded) && (eState == OMX_StateIdle))
		{
			if(omx_wfdfbdev_sink_component_Private->is_fbinit)
			{
				err = omx_wfdfbdev_sink_component_Deinit (openmaxStandComp);
				if (err != OMX_ErrorNone)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Video Sink Deinit Failed Error=%x\n", __func__, err);
					return err;
				}
				omx_wfdfbdev_sink_component_Private->is_fbinit = OMX_FALSE;
			}
		}
		else if((message->messageParam == OMX_StateIdle) && (omx_wfdfbdev_sink_component_Private->state == OMX_StateExecuting))
		{			
		}
	}

	// Execute the base message handling
	err = omx_base_component_MessageHandler (openmaxStandComp, message);

	return err;
}
