/******************************************************************************
*
*  (C)Copyright All Rights Reserved by Telechips Inc.
*                                   
*  This material is confidential and shall remain as such. 
*  Any unauthorized use, distribution, reproduction is strictly prohibited. 
*
*   FileName    : tcc_scaler_interface.c
*   Description : 
*   TCC Version 1.0
*   Copyright (c) Telechips, Inc.
*   ALL RIGHTS RESERVED
*******************************************************************************/

#include <main.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <sys/poll.h>
#include <stdio.h>
#include <pthread.h>  
#include "tcc_scaler_interface.h"
#include "main.h"

static int scaler_count = 0;
static pthread_mutex_t mutex_scaler;

#define SCALER_LOCK()		//pthread_mutex_lock(&mutex_scaler)
#define SCALER_UNLOCK()		//pthread_mutex_unlock(&mutex_scaler)


/*****************************************************************************
* Function Name :
******************************************************************************
* Desription  : tcc_video_scaler_open
* Parameter   : dev_name:device driver name
* Return	  : error:-1, others file descriptor
******************************************************************************/
int tcc_scaler_open(const char* dev_name)
{
	int fd = -1;

	if(dev_name != NULL)
	{
		fd = open(dev_name, O_RDWR | O_NDELAY);
		if(fd < 0)
			TCC_DEBUG_M(TCC_DEBUG_DRIVER, TCC_DEBUG_LEV_ERR, "can't open %s\n", dev_name);
	}

#if 0
	/* for thread safe */
	if(scaler_count == 0)
	{
		pthread_mutex_init(&mutex_scaler, NULL);
	}
	scaler_count++;
#endif
	return fd;
}

/*****************************************************************************
* Function Name :
******************************************************************************
* Desription  : tcc_video_scaler_execute
* Parameter   : fd:file descriptor

				src: addr_y, addr_u, addr_v is src address
					 x,y,w,h is input image size
					 w_x, w_y, w_w, w_h is window size (it can be used for zoom-in)

				dst:
					addr_y, addr_u, addr_v is dest address
					x,y,w,h is destination size

* Return	  : error:-1, others succeed
******************************************************************************/
int tcc_scaler_execute(const int fd, tcc_scaler_info_t* src, tcc_scaler_info_t* dst)
{
	int ret;
	struct pollfd poll_event[1];
	SCALER_TYPE scaler_info;

	if (fd < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_DRIVER, TCC_DEBUG_LEV_ERR, "fd error\n");
		return -1;
	}

	scaler_info.responsetype 		= SCALER_INTERRUPT;

	scaler_info.src_Yaddr			= src->addr_y;
	scaler_info.src_Uaddr			= src->addr_u;
	scaler_info.src_Vaddr			= src->addr_v;
	scaler_info.src_fmt				= src->format;
	scaler_info.src_ImgWidth 		= src->w;
	scaler_info.src_ImgHeight		= src->h;

	scaler_info.src_winLeft			= src->w_x;
	scaler_info.src_winTop			= src->w_y;
	scaler_info.src_winRight 		= src->w_x + src->w_w;
	scaler_info.src_winBottom		= src->w_y + src->w_h;

	scaler_info.dest_Yaddr			= dst->addr_y;
	scaler_info.dest_Uaddr			= dst->addr_u;
	scaler_info.dest_Vaddr			= dst->addr_v;

	scaler_info.dest_fmt 			= dst->format;
	scaler_info.dest_ImgWidth		= dst->w;
	scaler_info.dest_ImgHeight		= dst->h;

	// destnation window size Է.
	scaler_info.dest_winLeft 		= dst->w_x;
	scaler_info.dest_winTop			= dst->w_y;
 	scaler_info.dest_winRight		= dst->w_x + dst->w_w;
	scaler_info.dest_winBottom		= dst->w_y + dst->w_h;

  	scaler_info.viqe_onthefly = 0;
  	scaler_info.interlaced = 0;

  	SCALER_LOCK();
	if(ioctl(fd, TCC_SCALER_IOCTRL, &scaler_info) != 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_DRIVER, TCC_DEBUG_LEV_ERR, "scaler error\n");
		close(fd);
		goto RETURN_ERROR;
	}

	if(scaler_info.responsetype == SCALER_INTERRUPT)
	{
		memset(poll_event, 0, sizeof(poll_event));
		poll_event[0].fd = fd;
		poll_event[0].events = POLLIN;
		ret = poll((struct pollfd*)poll_event, 1, 400);

		if(ret < 0)
		{
			TCC_DEBUG_M(TCC_DEBUG_DRIVER, TCC_DEBUG_LEV_ERR, "m2m poll error %d\n", __LINE__);
			goto RETURN_ERROR;
		}
		else if(ret == 0)
		{
			TCC_DEBUG_M(TCC_DEBUG_DRIVER, TCC_DEBUG_LEV_ERR, "m2m poll error %d\n", __LINE__);
			goto RETURN_ERROR;
		}
		else if(ret > 0)
		{
			if (poll_event[0].revents & POLLERR)
			{
				TCC_DEBUG_M(TCC_DEBUG_DRIVER, TCC_DEBUG_LEV_ERR, "m2m poll error %d\n", __LINE__);
				goto RETURN_ERROR;
			}
		}
	}

	SCALER_UNLOCK();
	return 0;

RETURN_ERROR:
	SCALER_UNLOCK();
	return -1;
}

/*****************************************************************************
* Function Name :
******************************************************************************
* Desription  : tcc_video_scaler_close
* Parameter   : fd:file descriptor
* Return	  : error:-1, others succeed
******************************************************************************/
int tcc_scaler_close(const int fd)
{
	if(fd >= 0)
	{
		close(fd);
		return 0;
	}
	else
		TCC_DEBUG_M(TCC_DEBUG_DRIVER, TCC_DEBUG_LEV_ERR, "video scaler close error\n");

#if 0
	/* for thread safe */
	scaler_count--;
	if(scaler_count == 0)
	{
		pthread_mutex_destroy(&mutex_scaler);
	}
#endif
	return -1;
}

#ifdef SCALER_ARGB_TEST_APP
#include "tcc_fb_interface.h"
#include "pmap.h"
int main(int argc, char **argv)
{
	unsigned char *mem_fb = NULL;
	int mem_size;
	int fb0_handle;
	int scaler_handle;

	pmap_t fb_pmap;

	unsigned int frame_buffer_01;
	unsigned int frame_buffer_02;

	tcc_scaler_info_t scaler_src;
	tcc_scaler_info_t scaler_dst;

	FILE *write_file = NULL;

	struct fb_var_screeninfo info;
	struct fb_fix_screeninfo fix;

	if (argc != 2)
	{
		printf("Usage Error (%d) : ./TCC_XXXX_scaler_test [output file] \n", argc);

		return;
	}

	scaler_handle = tcc_scaler_open(TCC_SCALER1_DEV_NAME);
	fb0_handle = tcc_fb_open(TCC_FB0_DEV_NAME);

	write_file = fopen(argv[1], "w");

	tcc_fb_get_fscreen_info(fb0_handle, &fix);
	tcc_fb_get_vscreen_info(fb0_handle, &info);

	pmap_get_info(TCC_PMAP_FB_VIDEO, &fb_pmap);

	frame_buffer_01 = fb_pmap.base;
	frame_buffer_02 = fb_pmap.base + (info.xres * info.yres * (info.bits_per_pixel / 8));

	// src
	scaler_src.x = 0;
	scaler_src.y = 0;
	scaler_src.w = info.xres;
	scaler_src.h = info.yres;

	scaler_src.w_x = 0;
	scaler_src.w_y = 0;
	scaler_src.w_w = info.xres;
	scaler_src.w_h = info.yres;

	scaler_src.addr_y = (char *)frame_buffer_01;
	scaler_src.addr_u = NULL;
	scaler_src.addr_v = NULL;
	scaler_src.format = SCALER_ARGB8888;

	// dst
	scaler_dst.x = 0;
	scaler_dst.y = 0;
	scaler_dst.w = info.xres;
	scaler_dst.h = info.yres;

	scaler_dst.w_x = 0;
	scaler_dst.w_y = 0;
	scaler_dst.w_w = info.xres;
	scaler_dst.w_h = info.yres;

	scaler_dst.addr_y = (char *)frame_buffer_02;
	scaler_dst.addr_u = NULL;
	scaler_dst.addr_v = NULL;
	scaler_dst.format = SCALER_ARGB8888;

	mem_size = ((info.xres * info.yres) * (info.bits_per_pixel / 8)) * 2;
	mem_fb = (unsigned char *)mmap(0, mem_size, PROT_READ | PROT_WRITE, MAP_SHARED, fb0_handle, fix.mmio_start);
	memset(mem_fb, 0xFF, ((info.xres * info.yres) * (info.bits_per_pixel / 8)));

	if (tcc_scaler_execute(scaler_handle, &scaler_src, &scaler_dst) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_GUI, TCC_DEBUG_LEV_ERR, "SCALER ERROR  \n");
	}

	fwrite(mem_fb + ((info.xres * info.yres) * (info.bits_per_pixel / 8)), 1, ((info.xres * info.yres) * (info.bits_per_pixel / 8)), write_file);

	munmap(mem_fb, info.xres * (info.bits_per_pixel / 8) * info.yres * 2);
	fclose(write_file);
	tcc_fb_close(fb0_handle);
	tcc_scaler_close(scaler_handle);
}
#endif

#ifdef SCALER_TEST_APP && #ifndef SCALER_ARGB_TEST_APP
#define SCALER_PERFORMANCE_TEST
#define SCALER_TEST_FILE_OUT
//#define USING_SCALER_FILTER

#include "pmap.h"
#include "tcc_time.h"

#ifdef USING_SCALER_FILTER
/*********************************************************************
	SCALER FILTER
*********************************************************************/

/* FILT2D IOCTL code */
#define TCC_F2D_SET_MODE				0x301
#define TCC_F2D_SET_LPF_STRENGTH		0x302
#define TCC_F2D_SET_HPF_STRENGTH		0x303
#define TCC_F2D_SET_3x3WIN_COEFF_CH0	0x304
#define TCC_F2D_SET_3x3WIN_COEFF_CH1	0x305
#define TCC_F2D_SET_3x3WIN_COEFF_CH2	0x306
#define TCC_F2D_SET_DIV					0x307
#define TCC_F2D_SET_ENABLE				0x3FF
//

#define	PLANE0		0
#define	PLANE1		1
#define	PLANE2		2

#define	SMODE_00	0
#define	SMODE_01	1
#define	SMODE_02	2

#define ON		1
#define OFF		0

typedef struct {
	/* set detecting directon filter mode */
	// 0: LPF mode
	// 1: HPF mode
	unsigned int hpf0_en;		// channel.0
	unsigned int hpf1_en;		// channel.1
	unsigned int hpf2_en;		// channel.2

	/* set bypass mode */
	// 0: Normal mode
	// 1: Bypass mode
	unsigned int bypass0_en;	// channel.0
	unsigned int bypass1_en;	// channel.1
	unsigned int bypass2_en;	// channel.2

	/* set filter mode */
	// 0: detecting direction filter mode
	// 1: 3x3 window filter mode
	unsigned int simple0_mode;	// channel.0
	unsigned int simple1_mode;	// channel.1
	unsigned int simple2_mode;	// channel.2
} F2D_MODE_PARAM;

enum F2D_STRENGTH_TYPE {
	STRENGTH_1 = 0,
	STRENGTH_2 = 1,
	STRENGTH_3 = 2,
	STRENGTH_4 = 3,
	STRENGTH_5 = 4,
	STRENGTH_0 = 5,	// bypass
};

typedef struct {
	enum F2D_STRENGTH_TYPE ch0;
	enum F2D_STRENGTH_TYPE ch1;
	enum F2D_STRENGTH_TYPE ch2;
} F2D_FILT_STRENGTH_PARM;

typedef struct {
	unsigned int para[9];	// [0]-P00, [1]-P01, ..., [7]-P21, [8]-P22
	unsigned int cb;
} F2D_SCOEFF_PARAM;

typedef struct {
	unsigned int pos;
	unsigned int dtog;
	unsigned int den;
} F2D_DIV_PARAM;


void filt2d_coeff_lpf (int fd, unsigned int strength0, unsigned int strength1, unsigned int strength2)
{
	F2D_FILT_STRENGTH_PARM f2d_lpf_strength;

	f2d_lpf_strength.ch0 = strength0;
	f2d_lpf_strength.ch1 = strength1;
	f2d_lpf_strength.ch2 = strength2;

	ioctl(fd, TCC_F2D_SET_LPF_STRENGTH, &f2d_lpf_strength);
}

void filt2d_coeff_hpf (int fd, unsigned int strength0, unsigned int strength1, unsigned int strength2)
{
	F2D_FILT_STRENGTH_PARM f2d_hpf_strength;

	f2d_hpf_strength.ch0 = strength0;
	f2d_hpf_strength.ch1 = strength1;
	f2d_hpf_strength.ch2 = strength2;

	ioctl(fd, TCC_F2D_SET_HPF_STRENGTH, &f2d_hpf_strength);
}

void filt2d_coeff_simple (int fd, unsigned int plane, unsigned int *scoeff)
{
	F2D_SCOEFF_PARAM f2d_scoeff;

	memcpy(&f2d_scoeff, scoeff, sizeof(F2D_SCOEFF_PARAM));
	switch(plane)
	{
		case PLANE0:
			ioctl(fd, TCC_F2D_SET_3x3WIN_COEFF_CH0, &f2d_scoeff);
			break;
		case PLANE1:
			ioctl(fd, TCC_F2D_SET_3x3WIN_COEFF_CH1, &f2d_scoeff);
			break;
		case PLANE2:
			ioctl(fd, TCC_F2D_SET_3x3WIN_COEFF_CH2, &f2d_scoeff);
			break;
	}
}

void filt2d_mode (int fd, unsigned int hpf0_en, unsigned int hpf1_en, unsigned int hpf2_en,
						unsigned int bypass0_en, unsigned int bypass1_en, unsigned int bypass2_en,
						unsigned int simple0_mode, unsigned int simple1_mode, unsigned int simple2_mode)
{
	F2D_MODE_PARAM f2d_mode;

	f2d_mode.hpf0_en = hpf0_en;
	f2d_mode.hpf1_en = hpf1_en;
	f2d_mode.hpf2_en = hpf2_en;
	f2d_mode.bypass0_en = bypass0_en;
	f2d_mode.bypass1_en = bypass1_en;
	f2d_mode.bypass2_en = bypass2_en;
	f2d_mode.simple0_mode = simple0_mode;
	f2d_mode.simple1_mode = simple1_mode;
	f2d_mode.simple2_mode = simple2_mode;

	ioctl(fd, TCC_F2D_SET_MODE, &f2d_mode);
}

void filt2d_div	(int fd, uint pos, uint dtog, uint den)
{
	F2D_DIV_PARAM f2d_div;

	f2d_div.pos  = pos;
	f2d_div.dtog = dtog;
	f2d_div.den  = den;

	ioctl(fd, TCC_F2D_SET_DIV, &f2d_div);
}

void filt2d_enable (int fd, unsigned int enable)
{
	ioctl(fd, TCC_F2D_SET_ENABLE, &enable);
}

enum COEFFICIENT_TABLE
{
	FILTER_NOTHING,
	FILTER_BLUR_1,
	FILTER_BLUR_2,
	FILTER_SHARPEN_0,
	FILTER_SHARPEN_1,
	FILTER_EDGE_0,
	FILTER_EDGE_1,
	FILTER_EMBOSS,

	FILTER_MAX
};

/*simple 3x3 window filter coefficient example. */
static unsigned int f2d_simple_coeff[FILTER_MAX][10] = {
  //|P00  P01  P02| P10  P11  P12| P20  P21  P22|  CB
  	//FILTER_BLUR_0
	{0x0, 0x0, 0x0,
	 0x0, 0x1, 0x0,
	 0x0, 0x0, 0x0,  0x0},

	//FILTER_BLUR_1
	{0x1, 0x2, 0x1,
	 0x2, 0x4, 0x2,
	 0x1, 0x2, 0x1,  0x4},

	 //FILTER_BLUR_2
	{0x1, 0x1, 0x1,
	 0x1, 0x8, 0x1,
	 0x1, 0x1, 0x1,  0x4},

	//FILTER_SHARPEN_0
	{0, -1, 0,
	 -1, 5, -1,
	 0, -1, 0,  0},

	//FILTER_SHARPEN_1
	{-1, -1, -1,
	 -1, 8, -1,
	 -1, -1, -1,  0},

	//FILTER_EDGE_0
	{0, 0, 0,
	 -1, 1, 0,
	 0, 0, 0,  0},

	//FILTER_EDGE_1
	{0, 1, 0,
	 1, -4, 1,
	 0, 1, 0,  0},

	//FILTER_EMBOSS
	{-2, -1, 0,
	 -1, 1, 1,
	 0, 1, 2,  0},
};
//scaler filter end
/****************************************************************************************************/
#endif

int main(int argc, char** argv)
{
	int src_width, src_height;
	int dst_width, dst_height;

	unsigned char* scaler_ptr;
	unsigned char* scaler_dst_ptr;
	int phy_addr;
	pmap_t pmap_scaler;
	pmap_t pmap_scaler_dst;
	int memory_fd;
	FILE* input_fp = NULL;
	FILE* output_fp =  NULL;

	int scaler_fd;
	tcc_scaler_info_t src;
	tcc_scaler_info_t dst;

	int read_count;
	unsigned long long start_time;
	unsigned long long end_time;

	if(argc < 4)
	{
		printf("usage: TCC_892X_scaler_test source.yuv src_width src_height tar_width tar_height\n");
		return -1;
	}

	src_width = atoi(argv[2]);
	src_height = atoi(argv[3]);
	dst_width = atoi(argv[4]);
	dst_height = atoi(argv[5]);

	printf("TCC Scaler Test Program!! input:%s, %d x %d, target: %d x %d\n", argv[1], src_width, src_height, dst_width, dst_height);

	input_fp = fopen(argv[1], "r");
	if(input_fp == NULL)
	{
		printf("can't open %s\n", argv[1]);
		return -1;
	}

#ifdef SCALER_TEST_FILE_OUT
	output_fp = fopen("scaler_out.yuv", "w");
	if(output_fp == NULL)
	{
		printf("can't open output file\n");
		return -1;
	}
#endif

	memory_fd = open("/dev/mem", O_RDWR | O_NDELAY);
	if(memory_fd < 0)
	{
		printf("mem open erroe!!\n");
		return -1;
	}

	//get input memory information
	pmap_get_info("video", &pmap_scaler);
	pmap_scaler.size = pmap_scaler.size/2;
	scaler_ptr = (unsigned char*)mmap(0, pmap_scaler.size, PROT_READ | PROT_WRITE, MAP_SHARED, memory_fd, pmap_scaler.base);
	if(scaler_ptr == MAP_FAILED)
	{
		printf("mmap error!!\n");
		return -1;
	}

	//get output memory information
	pmap_scaler_dst.base = pmap_scaler.base + pmap_scaler.size;
	pmap_scaler_dst.size = pmap_scaler.size;
	scaler_dst_ptr = (unsigned char*)mmap(0, pmap_scaler_dst.size, PROT_READ | PROT_WRITE, MAP_SHARED, memory_fd, pmap_scaler_dst.base);
	if(scaler_dst_ptr == MAP_FAILED)
	{
		printf("mmap error!!\n");
		return -1;
	}

	//scaler driver open
	scaler_fd = tcc_scaler_open("/dev/scaler");
	if(scaler_fd < 0)
	{
		printf("scaler_fd open error!!\n");
		return -1;
	}

#ifdef USING_SCALER_FILTER

	/*
		Scaler Filter Configuration
	*/
	filt2d_coeff_lpf(scaler_fd, STRENGTH_3, STRENGTH_3, STRENGTH_3);
	filt2d_coeff_hpf(scaler_fd, STRENGTH_3, STRENGTH_3, STRENGTH_3);

	filt2d_coeff_simple(scaler_fd, PLANE0, f2d_simple_coeff[FILTER_SHARPEN_0]);
	filt2d_coeff_simple(scaler_fd, PLANE1, f2d_simple_coeff[FILTER_SHARPEN_0]);
	filt2d_coeff_simple(scaler_fd, PLANE2, f2d_simple_coeff[FILTER_SHARPEN_0]);

	filt2d_mode(scaler_fd, 	OFF, OFF, OFF,
							OFF, OFF, OFF,
							ON, ON, ON);

	//filt2d_div(scaler_fd, 0, 0, 2);
	filt2d_enable(scaler_fd, ON);
#endif

	//source setting
	phy_addr = pmap_scaler.base;
	src.addr_y = GET_ADDR_YUV42X_spY(phy_addr);
	src.addr_u = GET_ADDR_YUV42X_spU(src.addr_y, src_width, src_height);
	src.addr_v = GET_ADDR_YUV420_spV(src.addr_u, src_width, src_height);
	src.x = 0;
	src.y = 0;
	src.w = src_width;
	src.h = src_height;
	src.w_x = 0;
	src.w_y = 0;
	src.w_w = src_width;
	src.w_h = src_height;
	src.format = SCALER_YUV420_sp;

	//destination setting
	phy_addr = pmap_scaler_dst.base;
	dst.addr_y = GET_ADDR_YUV42X_spY(phy_addr);
	dst.addr_u = GET_ADDR_YUV42X_spU(dst.addr_y, dst_width, dst_height);
	dst.addr_v = GET_ADDR_YUV420_spV(dst.addr_u, dst_width, dst_height);
	dst.x = 0;
	dst.y = 0;
	dst.w = dst_width;
	dst.h = dst_height;
	dst.w_x = 0;
	dst.w_y = 0;
	dst.w_w = dst_width;
	dst.w_h = dst_height;
	dst.format = SCALER_YUV420_sp;

#ifdef SCALER_PERFORMANCE_TEST
	read_count = 0;
	while(1)
	{
		if(fread(scaler_ptr, 1, src_width * src_height * 3 / 2, input_fp) <= 0)
		{
			printf("read end!!\n");
			break;
		}

		start_time = TCC_GetTime_USec();
		//executing scaler
		if(tcc_scaler_execute(scaler_fd, &src, &dst) < 0)
		{
			printf("scaler error!!\n");
			return -1;
		}
		end_time = TCC_GetTime_USec();

		printf("count:%d, %d us\n",read_count, (int)( end_time - start_time));

#ifdef SCALER_TEST_FILE_OUT
		fwrite(scaler_dst_ptr, 1, (dst_width * dst_height * 3) / 2, output_fp);
#endif
		read_count++;
	}

	printf("read count = %d\n", read_count);
#else
	if(fread(scaler_ptr, 1, src_width * src_height * 3 / 2, input_fp) <= 0)
	{
		printf("read fail!!\n");
		return -1;
	}
#ifdef SCALER_TEST_FILE_OUT
	fwrite(scaler_dst_ptr, 1, (dst_width * dst_height * 3) / 2, output_fp);
#endif
#endif


#ifdef SCALER_TEST_FILE_OUT
	fclose(output_fp);
#endif

	fclose(input_fp);

	munmap(scaler_ptr, pmap_scaler.size);
	munmap(scaler_dst_ptr, pmap_scaler_dst.size);

	//scaler driver close
	tcc_scaler_close(scaler_fd);

	printf("test done!\n");
	return 0;
}

#endif

